;+
; NAME:
;     medcomb
;
; PURPOSE:
;     Combines data using a median. 
;
; CATEGORY:
;     Image Manipulation
;
; CALLING SEQUENCE:
;     medcomb,data,med,var,MASK=mask,UPDATE=update,WIDGET_ID=widget_id,$
;             CANCEL=cancel
;
; INPUTS:
;     data - Data array (1, 2 or 3D).
;
; OPTIONAL INPUTS:
;     None
;
; KEYWORD PARAMETERS:
;     MASK      - An optional mask of the same size as data
;                 identifying pixels to use in the combination.
;                 0=bad,1=good
;     UPDATE    - If set, the program will launch the Fanning
;                 showprogress widget.
;     WIDGET_ID - If given, a cancel button is added to the Fanning
;                 showprogress routine.  The widget blocks the
;                 WIDGET_ID, and checks for for a user cancel
;                 command.
;     CANCEL    - Set on return if there is an error. 
;
; OUTPUTS:
;     med - The median data
;     var - An estimate of the variance array (MAD^2 / N).
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     None
;
; PROCEDURE:
;     Combines the data together using a median.  An estimate of the 
;     error is is given by computing the Median Absolute Deviation (MAD), 
;     where MAD = 1.482*median(|x_i-x_med|), and then med_var = MAD^2/N.
;     
;     If data is 2D, then the median is down at each column.  Used to
;     combine spectra.
;
; EXAMPLE:
;     
;
;
; MODIFICATION HISTORY:
;     2001-04-22 - written by M. Cushing, Institute for Astronomy, UH
;     2003-05-27 - Added MASK keyword, M. Cushing
;-
pro medcomb,data,med,var,MASK=mask,UPDATE=update,WIDGET_ID=widget_id,$
            CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 1 then begin
    
    print, 'Syntax - mecomb,data,med,war,MASK=mask,UPDATE=update,$'
    print, '                WIDGET_ID=widget_id,CANCEL=cancel'
    cancel = 1
    return

endif
cancel = cpar('medcomb',data,1,'Data',[2,3,4,5],[1,2,3])
if cancel then return

;   Figure out dimension of data

s = size(data)

case s[0] of

    1: begin

        narr = n_elements(data)
        if n_elements(MASK) eq 0 then mask = intarr(narr)+1.0
        
    end
    2: begin

        ncols = s[1]
        narr  = s[2]
        med   = fltarr(ncols,/NOZERO)
        var   = fltarr(ncols,/NOZERO)
        if n_elements(MASK) eq 0 then mask = intarr(ncols,narr)+1.0


    end

    3: begin

        ncols = s[1]
        nrows = s[2]
        narr  = s[3]
        med   = fltarr(ncols,nrows,/NOZERO)
        var   = fltarr(ncols,nrows,/NOZERO)
        if n_elements(MASK) eq 0 then mask = intarr(ncols,nrows,narr)+1.0

    end

endcase

;  Perform combination.

if s[0] eq 1 then begin

    z = where(mask eq 1)
    med = median(data[z],/EVEN)
    var = (1.482*median( abs(data[z]-med), /EVEN))^2

endif else begin

;  Set up the Fanning showprogress object if requested.

    if keyword_set(UPDATE) then begin
        
        mkct
        cancelbutton = (n_elements(WIDGET_ID) ne 0) ? 1:0
        progressbar = obj_new('SHOWPROGRESS',widget_id,COLOR=2,$
                              CANCELBUTTON=cancelbutton,TITLE='Medcomb',$
                              MESSAGE='Median combining the images...')
        progressbar -> start
        
    endif
    
    for i = 0, ncols-1 do begin
        
        if s[0] eq 2 then begin
            
            z = where(mask[i,*] eq 1)
            med[i] = median(data[i,z],/EVEN)
            var[i] = (1.482*median( abs(data[i,z]-med[i]), /EVEN))^2
            
        endif else begin
            
            for j = 0, nrows-1 do begin
                
                z = where(mask[i,j,*] eq 1)
                med[i,j] = median(data[i,j,z], /EVEN)
                var[i,j] = (1.482*median( abs(data[i,j,z]-med[i,j]), /EVEN))^2

            endfor
            
        endelse
        if keyword_set(UPDATE) then begin
            
            if cancelbutton then begin
                
                cancel = progressBar->CheckCancel()
                if cancel then begin
                    
                    progressBar->Destroy
                    obj_destroy, progressbar
                    cancel = 1
                    return
                    
                endif
                
            endif
            percent = (i+1)*(100./float(ncols))
            progressbar->update, percent
            
        endif
        
    endfor

endelse
z = where(var eq 0,count)
if count ne 0 then var[z] = 1.
var = temporary(var)/float(narr)

if keyword_set(UPDATE) then begin

    progressbar-> destroy
    obj_destroy, progressbar

endif

end














