;+
; NAME:
;     writespec_xs
;    
; PURPOSE:
;     Writes extended source spectra to disk.
;
; CATEGORY:
;     Spectroscopy
;    
; CALLING SEQUENCE:
;     writespec_xs,spectra,xunits,yunits,fullpath,aimage,sky,flat,naps,orders,$
;                  start,stop,hdrinfo,appos,apradius,mode,slith_pix,slith_arc,$
;                  slitw_pix,slitw_arc,version,BGR=bgr,BGORDER=bgorder,$
;                  TRACEFILE=tracefile,ARCIMAGE=arcimage,DISP=disp,FITS=fits,$
;                  TEXT=text,RMS=rms,RES=res,LINEARITY=linearity,EXT2D=ext2d,$
;                  CANCEL=cancel
;    
; INPUTS:
;     spectra   - An array [*,3,naps*norders] array of spectra where
;                 array [*,0,0] = wavelengths
;                 array [*,1,0] = flux
;                 array [*,2,0] = error
;     xunits    - A string giving the units of array[*,0,0]
;     yunits    - A string giving the units of array[*,1,0]
;     fullpath  - The fullpath of the file to be written to
;     aimage    - A string of the name if the Aimage
;     sky       - A string of the sky image
;     flat      - A string of the flat field image
;     naps      - The number of apertures
;     orders    - An array of order numbers
;     start     - Start column
;     stop      - Stop column
;     hdrinfo   - A structure with FITS keywords and values
;     appos     - An array of aperture positions
;     apradius  - The aperture radius of the extraction
;     mode      - A string giving the mode name of the spectra
;     slith_pix - The slit length in pixels
;     slith_arc - The slit length in arcseconds
;     slitw_pix - The slit width in pixels
;     slitw_arc - The slit width in arcseconds
;     version   - Spextool version
;    
; OPTIONAL INPUTS:
;     None
;
; KEYWORD PARAMETERS:
;     BGR       - The background regions of the extraction 
;                 (see extractspec_xs.pro)
;     BGORDER   - The polynomial degree of the background fit
;     TRACEFILE - A string giving the name of the trace file used in 
;                 the extraction (if a trace file was used)
;     ARCIMAGE  - A string giving the name of the arc image      
;     DISP      - An array [nordesr*naps] of wavelength dispersion
;     FITS      - Write a FITS file
;     TEXT      - Write a text file
;     RMS       - An array [naps] of RMS values of the wavelength
;                 solution
;     RES       - The average resolution of the spectra      
;     LINEARITY - If set, a note will be put in the header that the 
;                 spectra were corrected for non-linearity
;     EXT2D     - Set to write the 2D extraction history instead of
;                 the 1D version
;     CANCEL   - Set on return if there is a problem
;     
; OUTPUTS:
;     Writes a FITS or text file to disk
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     None
;
; PROCEDURE:
;     Easy
;
; EXAMPLE:
;     
; MODIFICATION HISTORY:
;     2000-09-01 - Written by M. Cushing, Institute for Astronomy, UH
;     2002-10-09 - Added LINEARITY keyword
;     2003-04-10 - Added the appos input
;-
pro writespec_xs,spectra,xunits,yunits,fullpath,aimage,sky,flat,naps,orders,$
                 start,stop,hdrinfo,appos,apradius,mode,slith_pix,slith_arc,$
                 slitw_pix,slitw_arc,version,BGR=bgr,BGORDER=bgorder,$
                 TRACEFILE=tracefile,ARCIMAGE=arcimage,DISP=disp,FITS=fits,$
                 TEXT=text,RMS=rms,RES=res,LINEARITY=linearity,EXT2D=ext2d,$
                 CANCEL=cancel

cancel = 0

;  Check parameters

if n_params() lt 20 then begin
    
    print, 'Syntax - writespec_ps,spectra,xunits,yunits,fullpath,aimage,$'
    print, '                      sky,flat,naps,orders,start,stop,hdrinfo,$'
    print, '                      appos,apradius,mode,slith_pix,slith_arc,$'
    print, '                      slitw_pix,slitw_arc,version,$'
    print, '                      BGSTART=bgstart,BGWIDTH=bgwidth,$'
    print, '                      BGORDER=bgorder,TRACEFILE=tracefile,$'
    print, '                      ARCIMAGE=arcimage,DISP=disp,FITS=fits,$'
    print, '                      TEXT=text,RMS=rms,RES=res,$'
    print, '                      LINEARITY=linearity,EXT2D=ext2d,$'
    print, '                      CANCEL=cancel'

    cancel = 1
    return

endif
cancel = cpar('writespec_ps',spectra,1,'Spectra',[2,3,4,5],[2,3])
if cancel then return
cancel = cpar('writespec_ps',xunits,2,'Xunits',7,0)
if cancel then return
cancel = cpar('writespec_ps',yunits,3,'Yunits',7,0)
if cancel then return
cancel = cpar('writespec_ps',fullpath,4,'Fullpath',7,0) 
if cancel then return
cancel = cpar('writespec_ps',aimage,5,'Aimage',7,0) 
if cancel then return
cancel = cpar('writespec_ps',sky,6,'Sky',7,0)
if cancel then return
cancel = cpar('writespec_ps',flat,7,'Flat',7,0)
if cancel then return
cancel = cpar('writespec_ps',naps,8,'Naps',[2,3,4,5],0) 
if cancel then return
cancel = cpar('writespec_ps',orders,9,'Orders',[2,3,4,5],[0,1]) 
if cancel then return
cancel = cpar('writespec_ps',start,10,'Start',[2,3,4,5],0)
if cancel then return
cancel = cpar('writespec_ps',stop,11,'Stop',[2,3,4,5],0) 
if cancel then return
cancel = cpar('writespec_ps',hdrinfo,12,'Hdrinfo',8,[0,1])
if cancel then return
cancel = cpar('writespec_ps',appos,13,'Appos',[2,3,4,5],[0,1]) 
if cancel then return
cancel = cpar('writespec_ps',apradius,14,'Apradius',[2,3,4,5],[0,1]) 
if cancel then return
cancel = cpar('writespec_ps',mode,15,'Mode',7,0)
if cancel then return
cancel = cpar('writespec_ps',slith_pix,16,'Slith_pix',[2,3,4,5],0)
if cancel then return
cancel = cpar('writespec_ps',slith_arc,17,'Slith_arc',[2,3,4,5],0)
if cancel then return
cancel = cpar('writespec_ps',slitw_pix,18,'Slitw_pix',[2,3,4,5],0)
if cancel then return
cancel = cpar('writespec_ps',slitw_arc,19,'Slitw_arc',[2,3,4,5],0)
if cancel then return
cancel = cpar('writespec_ps',version,20,'Version',7,0)
if cancel then return

;  Make hdr with hdrinfo in it.

fxhmake,hdr,spectra

norders  = n_elements(orders)
ntags    = n_tags(hdrinfo)

names = tag_names(hdrinfo)
for i = 0, ntags - 1 do begin

    fxaddpar,hdr,names[i],hdrinfo.(i)

endfor

fxaddpar,hdr,'MODENAME',mode
fxaddpar,hdr,'AIMAGE',aimage
fxaddpar,hdr,'SKY',sky
fxaddpar,hdr,'FLAT',flat

fxaddpar,hdr,'NAPS',naps
fxaddpar,hdr,'NORDERS',norders
fxaddpar,hdr,'ORDERS',strjoin(strcompress(orders,/re),',')
fxaddpar,hdr,'START',start
fxaddpar,hdr,'STOP',stop
fxaddpar,hdr,'SLTH_PIX',slith_pix
fxaddpar,hdr,'SLTH_ARC',slith_arc
fxaddpar,hdr,'SLTW_PIX',slitw_pix
fxaddpar,hdr,'SLTW_ARC',slitw_arc

fxaddpar,hdr,'APPOS',strjoin(strtrim(appos,2),',')
fxaddpar,hdr,'APRADIUS',strjoin(strcompress(apradius,/re),',')
if n_elements(BGR) ne 0 then fxaddpar,hdr,'BGR',bgr
if n_elements(BGORDER) ne 0 then fxaddpar,hdr,'BGORDER',bgorder

if n_elements(TRACEFILE) ne 0 then fxaddpar,hdr,'TRACEFIL',tracefile
if n_elements(ARCIMAGE) ne 0 then fxaddpar,hdr,'ARCIMAGE',arcimage

fxaddpar,hdr,'XAXIS','Wavelength'
fxaddpar,hdr,'YAXIS','Flux'
fxaddpar,hdr,'XUNITS',xunits
fxaddpar,hdr,'YUNITS',yunits
fxaddpar,hdr,'VERSION',version

if n_elements(RES) ne 0 then fxaddpar,hdr,'RES',res

if n_elements(DISP) ne 0 then begin

    for i = 0,naps-1 do begin

        for j = 0, norders-1 do begin
            
            name = 'DPO'+string(orders[j],format='(i2.2)') + $
              'A'+string(i+1,format='(i1.1)')
            sxaddpar,hdr,name,disp[i+j*naps]

        endfor

    endfor

endif

if keyword_set(EXT2D) then begin

    history = 'The orders and apertures can be accessed as follows:  If the'+$ 
      ' user desires aperture Y in order X then lambda = [*,0,( X - '+$
      'min(orders))*naps + Y-1], flux = array[*,1:nspat,( X - '+$
      'min(orders))*naps + Y-1], error = array[*,(nspat+1):*,( X - '+$
      'min(orders))*naps + Y-1] where nspat=(NAXIS2-1)/2.'

endif else begin

    history = 'The orders and apertures can be accessed as follows:  If the'+$ 
      ' user desires aperture Y in order X then lambda = [*,0,( X - '+$
      'min(orders))*naps + Y-1], flux = array[*,1,( X - '+$
      'min(orders))*naps + Y-1], error = array[*,2,( X - '+$
      'min(orders))*naps + Y-1]'

endelse

if n_elements(RMS) ne 0 then begin

    history =history+'  The RMS in angstroms of the wavelength calibration '+$
      'is '+strjoin(strcompress(rms*10^4,/RE),', ')+' for aperture 1, 2, etc.'

endif

if keyword_set(LINEARITY) then history = history+'  The spectra have been '+$
  'corrected for non-linearity.'

length = strlen(history)
loop = ceil(float(length)/65.)
for i = 0, loop-1 do begin
    
    hist = strmid(history,65*i,65)
    fxaddpar,hdr,'HISTORY',hist
    
endfor

if keyword_set(FITS) then writefits,fullpath+'.fits',spectra,hdr

if keyword_set(TEXT) then begin

    npix = fxpar(hdr,'NAXIS1')
    openw,lun,fullpath+'.txt', /get_lun

    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]

    for i = 0, npix-1 do begin

        printf, lun,  strjoin( reform(spectra[i,*,*],3*naps*norders),'  ' )

    endfor
    close, lun
    free_lun, lun


endif

end
