;+
; NAME:
;     xcombspec
;    
; PURPOSE:
;     Combines SpeX spectra.
;    
; CATEGORY:
;     Widget
;
; CALLING SEQUENCE:
;     xcombspec
;    
; INPUTS:
;     None
;    
; OPTIONAL INPUTS:
;     None
;
; KEYWORD PARAMETERS:
;     None
;     
; OUTPUTS:
;     Writes a SpeX spectra FITS file to disk
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     Only works on SpeX spectra
;
; PROCEDURE:
;      Allows the user to remove bad spectra, and then scale the
;      spectra together.  They can then be combined using either
;      weighted mean, mean, or median.  The errors are obvious for
;      weighted mean and mean.  The error for the median is the median
;      absolute deviation MAD=1.482*median(|x_i-x_med|).
;
; EXAMPLE:
;     
; MODIFICATION HISTORY:
;     2000 - Written by M. Cushing, Institute for Astronomy, UH
;     2002-07-19 - Heavily modified by M. Cushing
;-

;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xcombspec_event, event

widget_control, event.id,  GET_UVALUE = uvalue
if uvalue eq 'Quit' then begin

    widget_control, event.top, /DESTROY
    goto, getout

endif

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, /HOURGLASS

case uvalue of

    'Aperture': begin

        state.r.ap = event.index
        xcombspec_plotupdate,state

    end

    'Combine Apertures': state.r.combineaps = event.value

    'Combine Spectra': xcombspec_combinespec,state

    'Combination Statistic':  begin

        state.r.combinestat = event.value
        sensitive = (event.value eq 'Mean') ? 1:0
        widget_control, state.w.meanerr_bg, SENSITIVE=sensitive

    end
    'File Name': begin

        path =dialog_pickfile(DIALOG_PARENT=state.w.xcombspec_base,$
                              /MUST_EXIST)

        if path ne '' then begin

            widget_control,state.w.filename_fld[1],SET_VALUE=path

        endif

    end

    'Help': xcombspec_help,state

    'Input Mode': begin

        widget_control, state.w.manual_base,MAP=0
        widget_control, state.w.file_base,MAP=0
        state.r.inputmode = event.value
        if event.value eq 'File' then widget_control, state.w.file_base,/MAP
        if event.value eq 'Manual' then widget_control, state.w.manual_base,$
          /MAP


    end

    'Input Prefix': setfocus, state.w.specfiles_fld

    'Load Spectra': xcombspec_loadspec,state

    'Make Mask': xcombspec_mkmask,state,event.index

    'Mean Error': state.r.meanerror = event.value

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select

    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTOR,DIALOG_PARENT=state.w.xcombspec_base,$
                              TITLE='Select Path',/MUST_EXIST)
        
        if path ne '' then begin

            path = cpath(path,WIDGET_ID=state.w.xcombspec_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld

        endif
    end

    'Readmode': begin

        widget_control, state.w.inprefix_fld[0],  SENSITIVE=0
        if event.value eq 'Filename' then begin
            
            state.r.filereadmode = event.value
            widget_control, state.w.inprefix_fld[0], SENSITIVE=0
            setfocus,state.w.specfiles_fld
            
        endif else begin
            
            state.r.filereadmode = event.value
            widget_control, state.w.inprefix_fld[0], /SENSITIVE
            setfocus,state.w.inprefix_fld

        endelse

    end

    'Scale Order': xcombspec_scalespec,state,event.index

    'Spectra Files Button': begin
                
        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        if path ne '' then path = cpath(path,WIDGET_ID=state.w.xcombspec_base,$
                                        CANCEL=cancel)

        fullpath = dialog_pickfile(DIALOG_PARENT=state.w.xcombspec_base,$
                                   PATH=path,/MUST_EXIST,FILTER='*.fits',$
                                   /MULTIPLE_FILES)
        
        case (size(fullpath))[1] of 

            1: begin

                if fullpath[0] ne '' then begin
                    
                    widget_control,state.w.specfiles_fld[1],$
                      SET_VALUE=strmid(fullpath[0],$
                                       strpos(fullpath,'/',/REVERSE_S)+1)
                    setfocus, state.w.outfile_fld
                    
                endif

            end

            else: begin

                for i =0,(size(fullpath))[1]-1 do begin
 
                    tmp = strmid(fullpath[i],strpos(fullpath[i],'/',$
                                                    /REVERSE_S)+1)
                    arr = (i eq 0) ? tmp:[arr,tmp]

                endfor
                widget_control,state.w.specfiles_fld[1],$
                  SET_VALUE=strjoin(arr,',',/SINGLE)

            end

        endcase

    end

    'Spectra Files Field': setfocus,state.w.outfile_fld

    else:


endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xcombspec_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
pro xcombspec_resize,event

widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, event.id,  GET_UVALUE = uvalue

widget_control, state.w.xcombspec_base, TLB_GET_SIZE=size

state.p.plotwinsize[0]=size[0]-state.p.buffer[0]
state.p.scrollsize[0] =state.p.plotwinsize[0]


state.p.scrollsize[1]=size[1]-state.p.buffer[1]
state.p.plotwinsize[1] = state.p.scrollsize[1] > $
  state.p.pixpp*state.r.norders*(1 > (2-state.r.norders*0.1))

;  Modify plot window according to the number of orders

widget_control, state.w.col2_base, UPDATE=0
widget_control, state.w.plotwin, /DESTROY

state.w.plotwin = widget_draw(state.w.col2_base,$
                              XSIZE=state.p.plotwinsize[0],$
                              YSIZE=state.p.plotwinsize[1],$
                              X_SCROLL_SIZE=state.p.scrollsize[0],$
                              Y_SCROLL_SIZE=state.p.scrollsize[1],$
                              UVALUE='Plot Window',$
                              /BUTTON_EVENTS,$
                              EVENT_PRO='xcombspec_selectspec')

widget_control, state.w.col2_base, UPDATE=1

wdelete, state.p.pixmap_wid
window, /FREE, /PIXMAP,XSIZE=state.p.plotwinsize[0],$
  YSIZE=state.p.plotwinsize[1]
state.p.pixmap_wid = !d.window


xcombspec_plotupdate,state

widget_control, state.w.xcombspec_base, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xcombspec_selectspec,event

widget_control, /HOURGLASS
widget_control, event.top, GET_UVALUE = state, /NO_COPY

spc = *state.d.wspec
mask = *state.r.mask

idx = floor(event.y/float(state.p.plotwinsize[1])*state.r.norders)

xselectspec,reform(spc[state.r.ap].(idx)[*,0,0]),$
  reform(spc[state.r.ap].(idx)[*,1,*]),omask,$
  GROUP_LEADER=state.w.xcombspec_base,IMASK=mask,CANCEL=cancel
if cancel then goto,cont

*state.r.mask = omask
xcombspec_scalespec,state,0
widget_control, state.w.scaleorder_dl,SET_DROPLIST_SELECT=0

delvarx,spc

cont:

widget_control, event.top, SET_UVALUE=state, /NO_COPY

end
;
;******************************************************************************
;
pro xcombspechelp_event,event

widget_control, event.top, /DESTROY

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xcombspec_cleanup,xcombspec_base

widget_control, xcombspec_base, GET_UVALUE = state, /NO_COPY
if n_elements(state) ne 0 then begin
    
    ptr_free, state.r.files
    ptr_free, state.r.mask
    ptr_free, state.r.orders
    ptr_free, state.d.ospec
    
    ptr_free, state.d.wspec
    ptr_free, state.d.hdrinfo
    
endif 
state = 0B

end
;
;******************************************************************************
;
pro xcombspec_combinespec,state

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return

outname = cfld(state.w.outfile_fld,7,CANCEL=cancel,/EMPTY)
if cancel then return

spc = *state.d.wspec
mask = *state.r.mask

sspec = (state.r.combineaps eq 'Yes') ? $
  fltarr(state.r.npix,3,state.r.norders):$
  fltarr(state.r.npix,3,state.r.norders*state.r.naps)

naps = (state.r.combineaps eq 'No') ? state.r.naps:1.

z = where(mask eq 1,count)
for i = 0, naps-1 do begin

    for j = 0, state.r.norders-1 do begin

        case state.r.combinestat of 
            
            'Sum': begin
                
                mean = total(reform(spc[i].(j)[*,1,z]),2)
                mvar = total(reform(spc[i].(j)[*,2,z])^2,2)

            end

            'Weighted Mean': meancomb,reform(spc[i].(j)[*,1,z]),mean,mvar,$
              DATAVAR=reform((spc[i].(j)[*,2,z]))^2,CANCEL=cancel
            
            'Mean': begin

                meancomb,reform(spc[i].(j)[*,1,z]),mean,mvar,CANCEL=cancel
                if state.r.meanerror then mvar = temporary(mvar)/float(count)

            end
            'Median': medcomb,reform(spc[i].(j)[*,1,z]),mean,mvar,$
              CANCEL=cancel
            
        endcase
        
        sspec[*,0,j*naps+i] = reform(spc[i].(j)[*,0,0])
        sspec[*,1,j*naps+i] = mean
        sspec[*,2,j*naps+i] = sqrt(mvar)
       
    endfor

endfor

;  Make new hdr.

avehdr = avehdrs(*state.d.hdrinfo,TIME_OBS=state.r.time,$
                 POSANGLE=state.r.posangle,HA=state.r.ha,$
                 AIRMASS=state.r.airmass,CANCEL=cancel)

avehdr.NAPS = (state.r.combineaps eq 'Yes') ? 1:state.r.naps


history='The spectra '+strjoin((*state.r.files)[z],', ')+$
  ' were combined using a '+state.r.combinestat+' to create these spectra.'

fxhmake,newhdr,sspec

ntags    = n_tags(avehdr)
names = tag_names(avehdr)
for i = 0, ntags - 1 do begin

    if names[i] eq 'HISTORY' then begin

        for j = 0, ceil(float(strlen(avehdr.(i)))/70.)-1 do begin
            
            hist = strmid(avehdr.(i),70*j,70)
            fxaddpar,newhdr,'HISTORY',hist
            
        endfor
       
    endif else fxaddpar,newhdr,names[i],avehdr.(i)

endfor

for i = 0, ceil(float(strlen(history))/70.)-1 do begin
    
    hist = strmid(history,70*i,70)
    fxaddpar,newhdr,'HISTORY',hist
    
endfor

xvspec,sspec,long( strsplit(avehdr.ORDERS,',',/extrac) ),avehdr.NAPS,$
  XUNITS=xunits,YUNITS=yunits,FILE=outname+'.fits',HDR=newhdr

if state.r.fitsoutput then writefits,path+outname+'.fits',sspec,newhdr

if state.r.textoutput then begin $

    t = (state.r.combineaps eq 'Yes') ? 1:state.r.naps
    npix = fxpar(newhdr,'NAXIS1')

    openw,lun,path+outname+'.txt', /GET_LUN

    for i = 0, n_elements(newhdr)-1 do printf, lun, newhdr[i]

    for i = 0, npix[0]-1 do begin

        printf, lun,  strjoin( reform(sspec[i,*,*],3*t*state.r.norders),'  ' )

    endfor
    close, lun
    free_lun, lun

endif

end
;
;******************************************************************************
;
pro xcombspec_help,state


getfonts,buttonfont

if not xregistered('xcombspec_help') then begin

    help_base = widget_base(GROUP_LEADER=state.w.xcombspec_base,$
                            /COLUMN, $
                            TITLE='Xcombspec Help')

    openr, lun, filepath('xcombspec_helpfile.txt',$
                         ROOT_DIR=state.r.packagepath,SUBDIR='helpfiles'),$
      /get_lun
    nlines = numlines(filepath('xcombspec_helpfile.txt',$
                               ROOT_DIR=state.r.packagepath,$
                               SUBDIR='helpfiles'))
    array = strarr(nlines)
    readf, lun, array
    free_lun, lun

    help_text = widget_text(help_base, $
                            /scroll, $
                            VALUE=array, $
                            XSIZE=80, $
                            YSIZE=24)
    
    quit = widget_button(help_base,$
                         VALUE='Done',$
                         FONT=buttonfont,$
                         UVALUE='Done')
      
    centertlb,help_base

    widget_control, help_base, /REALIZE
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xcombspec_help', $
      help_base, $
      /NO_BLOCK,$
      EVENT_HANDLER='xcombspechelp_event'
    
endif

end
;
;******************************************************************************
;
pro xcombspec_loadspec,state

;  Construct full file names

if state.r.inputmode eq 'Manual' then begin
    
;  Get path and file names
    
    path = cfld(state.w.path_fld,7,CANCEL=cancel)
    if cancel then return

    files = cfld(state.w.specfiles_fld,7,/EMPTY,CANCEL=cancel)
    if cancel then return
    
;  Construct array of full file names
    
    index    = (state.r.filereadmode eq 'Index') ? 1:0
    filename = (state.r.filereadmode eq 'Filename') ? 1:0
    if index then prefix = cfld(state.w.inprefix_fld,7,/EMPTY,CANCEL=cancel)
    if cancel then return
    
    files = fsextract(files,INDEX=index,FILENAME=filename,CANCEL=cancel)
    if cancel then return
    
    fullpaths = mkfullpath(path,files,INDEX=index,FILENAME=filename,$
                           NI=state.r.nint,$
                           PREFIX=prefix,SUFFIX='.fits',$
                           WIDGET_ID=state.w.xcombspec_base,/EXIST,$
                           CANCEL=cancel)
    if cancel then return
    
endif else begin

    filename = cfld(state.w.filename_fld,7,CANCEL=cancel)
    if cancel then return    
    nl = numlines(filename)
    fullpaths = strarr(nl)
    openr,lun, filename,/GET_LUN
    line = ' '
    for i = 0, nl-1 do begin

        readf, lun, line
        fullpaths[i] = strtrim(line,2)

    endfor
    free_lun, lun
    path = ''
    
endelse

;  Read the first spectrum to get initial data

first = readfits(fullpaths[0],hdr)

state.r.npix     = fxpar(hdr,'NAXIS1')
state.r.norders  = fxpar(hdr,'NORDERS')
state.r.naps     = fxpar(hdr,'NAPS')
state.p.xunits   = fxpar(hdr,'XUNITS')
state.p.yunits   = fxpar(hdr,'YUNITS')
*state.r.orders  = long( strsplit( fxpar(hdr,'ORDERS'), ',', /EXTRACT) )
state.r.nfiles   = n_elements(fullpaths)

;  Modify the scale base

widget_control, state.w.scaleorder_dl,$
  SET_VALUE=['None',string(*state.r.orders,FORMAT='(i2.2)')]

;  Modify plot window according to the number of orders

state.p.plotwinsize[1] = state.p.scrollsize[1] > $
  state.p.pixpp*state.r.norders*(1 > (2-state.r.norders*0.1))

widget_control, state.w.col2_base, UPDATE=0
widget_control, state.w.plotwin, /DESTROY

state.w.plotwin = widget_draw(state.w.col2_base,$
                              XSIZE=state.p.plotwinsize[0],$
                              YSIZE=state.p.plotwinsize[1],$
                              X_SCROLL_SIZE=state.p.scrollsize[0],$
                              Y_SCROLL_SIZE=state.p.scrollsize[1],$
                              UVALUE='Plot Window',$
                              /BUTTON_EVENTS,$
                              EVENT_PRO='xcombspec_selectspec')

widget_control, state.w.col2_base, UPDATE=1
widget_control, state.w.plotwin, get_value = x
state.p.plotwin_wid = x

wdelete, state.p.pixmap_wid
window, /FREE, /PIXMAP,XSIZE=state.p.plotwinsize[0],$
  YSIZE=state.p.plotwinsize[1]
state.p.pixmap_wid = !d.window


;  Combining apertures?

state.r.ap = 0
if state.r.combineaps eq 'No' then begin

    value   = 'Aperture '+strcompress(indgen(state.r.naps)+1, /RE)
    widget_control, state.w.aperture_dl, SET_VALUE=value
    widget_control, state.w.aperture_dl, /SENSITIVE
    array   = fltarr(state.r.npix,3,state.r.nfiles)
    pmask   = intarr(state.r.nfiles)+1
    ds_naps = state.r.naps 
    
endif else begin

    array   = fltarr(state.r.npix,3,state.r.nfiles*state.r.naps)
    pmask   = intarr(state.r.naps*state.r.nfiles)+1
    ds_naps = 1 
    widget_control, state.w.aperture_dl, SENSITIVE=0

endelse

;  Create data structure for a single aperture

key   = 'Order'+string(00,FORMAT='(i2.2)')
spc   = create_struct(key,array)

for i = 1, state.r.norders-1 do begin

    key = 'Order'+string(i,FORMAT='(i2.2)')
    spc = create_struct(spc,key,array)

endfor

hdrinfo = replicate(gethdrinfo(hdr),state.r.nfiles)

;  If there are more than 1 apertures replicate the data structure.

spc   = replicate(spc,ds_naps)

;  Load the data

for i = 0, state.r.nfiles-1 do begin

    data = readfits(fullpaths[i],hdr)
    copy_struct_inx,gethdrinfo(hdr),hdrinfo,index_to=i

    for j = 0, state.r.norders-1 do begin

        if state.r.combineaps ne 'Yes' then begin

            for k = 0, state.r.naps-1 do begin

                interpspec,data[*,0,j*state.r.naps+k],$
                  data[*,1,j*state.r.naps+k],first[*,0,j*state.r.naps+k],$
                  newflux,newerror,YAERROR=data[*,2,j*state.r.naps+k]

                spc[k].(j)[*,0,i] = first[*,0,j*state.r.naps+k]
                spc[k].(j)[*,1,i] = newflux
                spc[k].(j)[*,2,i] = newerror              
                
            endfor

        endif else begin

            for k = 0,state.r.naps-1 do begin

                interpspec,data[*,0,j*state.r.naps+k],$
                  data[*,1,j*state.r.naps+k],first[*,0,j*state.r.naps+k],$
                  newflux,newerror,YAERROR=data[*,2,j*state.r.naps+k]

                spc[0].(j)[*,0,i*state.r.naps+k] = first[*,0,j*state.r.naps+k]
                spc[0].(j)[*,1,i*state.r.naps+k] = newflux
                spc[0].(j)[*,2,i*state.r.naps+k] = newerror              

            endfor

        endelse

    endfor

endfor

*state.d.ospec    = spc
*state.d.wspec    = spc
*state.r.mask     = pmask
*state.d.hdrinfo  = hdrinfo

xcombspec_plotupdate,state

;  Create filename string for output FITS header later

for i = 0,state.r.nfiles-1 do begin

    file = strmid(fullpaths[i],strpos(fullpaths[i],'/',/REVERSE_S)+1)
    sfile = (i eq 0) ? file:[sfile,file]

endfor
*state.r.files = sfile

;  Unfreeze the widget

widget_control, state.w.scaleorder_dl, /SENSITIVE


end
;
;******************************************************************************
;
pro xcombspec_mkmask,state

if index eq 0 then begin

    *state.d.wspec = *state.d.ospec

endif else begin

;    spc  = *state.d.ospec
;    mask = *state.r.mask
;    naps = (state.r.combineaps eq 'No') ? state.r.naps:1.
;    idxs = intarr(naps)
    
;    zorder = total(where(*state.r.orders eq (*state.r.orders)[index-1]))
    
;    xscalespec,reform(spc[state.r.ap].(zorder)[*,0,0]),$
;      reform(spc[state.r.ap].(zorder)[*,1,*]),mask,scales,$
;      wrange,GROUP_LEADER=state.w.xcombspec_base,$
;      XUNITS=state.p.xunits,YUNITS=state.p.yunits,CANCEL=cancel
;    if cancel then return

;    for i = 0, state.r.norders-1 do begin

;        s = reform(spc[state.r.ap].(i)[*,1,*])
;        e = reform(spc[state.r.ap].(i)[*,2,*])

;        for k = 0, n_elements(mask)-1 do begin
            
;            s[*,k] = scales[k] * s[*,k]
;            e[*,k] = sqrt(scales[k]^2 * e[*,k]^2)
            
;        endfor
        
;        spc[state.r.ap].(i)[*,1,*] = s
;        spc[state.r.ap].(i)[*,2,*] = e        
;            
;    endfor
;    *state.d.wspec = spc

endelse
;xcombspec_plotupdate,state



end
;
;******************************************************************************
;
pro xcombspec_plotspec,state

spc  = *state.d.wspec
mask = *state.r.mask

!p.multi[0] = state.r.norders
!p.multi[2] = state.r.norders

ds_naps = (state.r.combineaps eq 'Yes') ? state.r.naps:1
xtitle='!5Wavelength ('+strtrim(state.p.xunits,2)+')'
ytitle='!5Flux ('+strtrim(state.p.yunits,2)+')'

for i = 0, state.r.norders-1 do begin

    j     = state.r.norders-1-i
    title = 'Order '+string((*state.r.orders)[j],FORMAT='(i2.2)')
    
;  Get plot range.
    
    medcomb,spc[state.r.ap].(j)[*,1,*],med
    yrange = [0.4*min(med,/NAN,MAX=max),1.5*max]
    
;  Plot spectra    

    plot,spc[state.r.ap].(j)[*,0,0],spc[state.r.ap].(j)[*,1,0],$
      /XSTY,/YSTY,YRANGE=yrange,/NODATA,CHARSIZE=2,TITLE=title,$
      XTITLE=xtitle,YTITLE=ytitle

    for k = 0, ds_naps*state.r.nfiles-1 do begin

        if mask[k] eq 1 then $
        oplot,spc[state.r.ap].(j)[*,0,k],spc[state.r.ap].(j)[*,1,k],$
          COLOR=state.r.colors[k],LINESTYLE=state.r.lines[k]
        
    endfor
    
endfor
!p.multi=0
delvarx,spc

end
;
;******************************************************************************
;
pro xcombspec_plotupdate,state

wset,state.p.pixmap_wid
erase

xcombspec_plotspec,state

wset, state.p.plotwin_wid
erase
device, COPY=[0,0,state.p.plotwinsize[0],state.p.plotwinsize[1],0,0,$
                  state.p.pixmap_wid]

end
;
;******************************************************************************
;
pro xcombspec_scalespec,state,index

if index eq 0 then begin

    *state.d.wspec = *state.d.ospec

endif else begin

    spc  = *state.d.ospec
    mask = *state.r.mask
    naps = (state.r.combineaps eq 'No') ? state.r.naps:1.
    idxs = intarr(naps)
    
    zorder = total(where(*state.r.orders eq (*state.r.orders)[index-1]))
    
    xscalespec,reform(spc[state.r.ap].(zorder)[*,0,0]),$
      reform(spc[state.r.ap].(zorder)[*,1,*]),mask,scales,$
      wrange,GROUP_LEADER=state.w.xcombspec_base,$
      XUNITS=state.p.xunits,YUNITS=state.p.yunits,CANCEL=cancel
    if cancel then return

    for i = 0, state.r.norders-1 do begin

        s = reform(spc[state.r.ap].(i)[*,1,*])
        e = reform(spc[state.r.ap].(i)[*,2,*])

        for k = 0, n_elements(mask)-1 do begin
            
            s[*,k] = scales[k] * s[*,k]
            e[*,k] = sqrt(scales[k]^2 * e[*,k]^2)
            
        endfor
        
        spc[state.r.ap].(i)[*,1,*] = s
        spc[state.r.ap].(i)[*,2,*] = e        
            
    endfor
    *state.d.wspec = spc

endelse
xcombspec_plotupdate,state

end
;
;******************************************************************************
;
pro xcombspec,instrfile

mkct
device, RETAIN=2

;  Determine the instrument in use, default is SpeX.

if n_elements(instrfile) eq 0 then instrfile = 'SpeX.dat'

last   = strpos(!path,'Spextool')
first  = strpos(!path,':',last,/REVERSE_SEARCH)
result = strmid(!path,first+1,last-first+7)

path = cpath(result,CANCEL=cancel)
if cancel then return

readinstrfile,path+'data/'+instrfile,instr,irafname,gain,readnoise,itime,$
  coadds,ndrs,slowcnt,readtime,time,posangle,ha,airmass,nint,bdpxmk,keywords,$
  ncols,nrows,CANCEL=cancel

;  Set the fonts

getfonts,buttonfont,textfont

;  Build three structures which will hold important info.
;  w - contains info pertaining to widget operations.
;  r - contains info pertaining to the reduction process.
;  d - contains all of the data.

w = {aperture_dl:0L,$
     col2_base:0L,$
     manual_base:0L,$
     meanerr_bg:0L,$
     medscale:0L,$
     path_fld:[0L,0L],$
     file_base:0L,$
     filename_fld:[0L,0L],$
     inprefix_fld:[0L,0L],$
     keyboard:0L,$
     mask_dl:0L,$
     outfile_fld:[0L,0L],$
     plotwin:0L,$
     scaleorder_dl:0L,$
     scalerange_fld:[0L,0L],$
     specfiles_fld:[0L,0L],$
     xcombspec_base:0L}

r = {absscalerange:[!values.f_nan,!values.f_nan],$
     airmass:airmass,$
     ap:0,$
     ha:ha,$
     colors:[1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,$
             1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,$
             1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,$
             1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,$
             1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,$
             1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16],$
     combineaps:'No',$
     combinestat:'Median',$
     files:ptr_new(fltarr(2)),$
     filereadmode:'Index',$
     meanerror:1,$
     nint:nint,$
     packagepath:path,$
     fitsoutput:1,$
     ifileformat:'FITS',$
     inputmode:'Manual',$
     lines:[0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,$
            1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,$
            2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,$
            3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,$
            4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,$
            5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5],$
     mask:ptr_new(2),$
     naps:0,$
     nfiles:0,$
     norders:0,$
     npix:0,$
     orders:ptr_new(2),$     
     posangle:posangle,$
     scaleorder:0L,$
     scalerange:[!values.f_nan,!values.f_nan],$
     textoutput:0,$
     time:time}

p = {buffer:[0,0],$
     pixpp:((get_screen_size())[1]-150)/float(6),$
     pixmap_wid:0L,$
     plotwin_wid:0,$
     plotwinsize:[512,590],$
     scrollsize:[512,590],$
     xunits:'',$
     yunits:''}

d = {ospec:ptr_new(fltarr(2)),$
     wspec:ptr_new(fltarr(2)),$
     hdrinfo:ptr_new(strarr(2))}

state = {w:w,r:r,d:d,p:p}

;  Build the widget.

state.w.xcombspec_base = widget_base(TITLE='xcombspec', $
                                     /COLUMN,$
                                     /TLB_SIZE_EVENTS)

   quit_button = widget_button(state.w.xcombspec_base,$
                               FONT=buttonfont,$
                               VALUE='Quit',$
                               EVENT_PRO='xcombspec_event',$
                               UVALUE='Quit')

   state.w.keyboard = widget_text(state.w.xcombspec_base, $
                                  /ALL_EVENTS, $
                                  SCR_XSIZE=1, $
                                  SCR_YSIZE=1, $
                                  UVALUE='Keyboard', $
                                  EVENT_PRO='xcombspec_event',$
                                  VALUE='')

   row_base = widget_base(state.w.xcombspec_base,$
                          /ROW)

      col1_base = widget_base(row_base,$
                              /COLUMN)

         box1_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 EVENT_PRO='xcombspec_event',$
                                 /FRAME)

            bg = cw_bgroup(box1_base,$
                           ['Manual','File'],$
                           /ROW,$
                           LABEL_LEFT='Input Mode:',$
                           /RETURN_NAME,$
                           /NO_RELEASE,$
                           UVALUE='Input Mode',$
                           FONT=buttonfont,$
                           /EXCLUSIVE,$
                           SET_VALUE=0)

            blank = widget_base(box1_base,$
                                /BASE_ALIGN_CENTER)

               state.w.manual_base = widget_base(blank,$
                                                 /COLUMN,$
                                                 /BASE_ALIGN_LEFT)

                  row = widget_base(state.w.manual_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)
                  
                     button = widget_button(row,$
                                            FONT=buttonfont,$
                                            VALUE='Path',$
                                            UVALUE='Path Button',$
                                            EVENT_PRO='xcombspec_event')
                     
                     field = coyote_field2(row,$
                                           LABELFONT=buttonfont,$
                                           FIELDFONT=textfont,$
                                           TITLE=':',$
                                           UVALUE='Path Field',$
                                           XSIZE=25,$
                                           EVENT_PRO='xcombspec_event',$
                                           /CR_ONLY,$
                                           TEXTID=textid)
                     state.w.path_fld = [field,textid]               
               
                  bg = cw_bgroup(state.w.manual_base,$
                                 ['Filename','Index'],$
                                 /ROW,$
                                 LABEL_LEFT='File Read Mode:',$
                                 /RETURN_NAME,$
                                 /NO_RELEASE,$
                                 UVALUE='Readmode',$
                                 FONT=buttonfont,$
                                 /EXCLUSIVE,$
                                 SET_VALUE=1)
            
                  field = coyote_field2(state.w.manual_base,$
                                        LABELFONT=buttonfont,$
                                        FIELDFONT=textfont,$
                                        TITLE='Input Prefix:',$
                                        UVALUE='Input Prefix',$
                                        XSIZE=25,$
                                        VALUE='spectra',$
                                        EVENT_PRO='xcombspec_event',$
                                        /CR_oNLY,$
                                        TEXTID=textid)
                  state.w.inprefix_fld = [field,textid]
                  
                  row = widget_base(state.w.manual_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)
                  
                     button = widget_button(row,$
                                            FONT=buttonfont,$
                                            VALUE='Spectra Files',$
                                            UVALUE='Spectra Files Button',$
                                            EVENT_PRO='xcombspec_event')
                     
                     field = coyote_field2(row,$
                                           LABELFONT=buttonfont,$
                                           FIELDFONT=textfont,$
                                           TITLE=':',$
                                           UVALUE='Spectra Files Field',$
                                           XSIZE=25,$
                                           EVENT_PRO='xcombspec_event',$
                                           /CR_ONLY,$
                                           TEXTID=textid)
                     state.w.specfiles_fld = [field,textid]
                     
               state.w.file_base = widget_base(blank,$
                                               /COLUMN,$
                                               /BASE_ALIGN_CENTER)

                  row = widget_base(state.w.file_base,$
                                    /ROW,$
                                    /BASE_ALIGN_CENTER)
                  
                     button = widget_button(row,$
                                            FONT=buttonfont,$
                                            VALUE='File Name',$
                                            UVALUE='File Name',$
                                            EVENT_PRO='xcombspec_event')
                     
                     field = coyote_field2(row,$
                                           LABELFONT=buttonfont,$
                                           FIELDFONT=textfont,$
                                           TITLE=':',$
                                           UVALUE='File Name Field',$
                                           XSIZE=25,$
                                           EVENT_PRO='xcombspec_event',$
                                           /CR_ONLY,$
                                           TEXTID=textid)
                     state.w.filename_fld = [field,textid]
                     
            combineap_bg = cw_bgroup(box1_base,$
                                     FONT=buttonfont,$
                                     ['Yes','No'],$
                                     /ROW,$
                                     /RETURN_NAME,$
                                     /NO_RELEASE,$
                                     /EXCLUSIVE,$
                                     LABEL_LEFT='Combine Apertures:',$
                                     UVALUE='Combine Apertures',$
                                     SET_VALUE=1)


            load = widget_button(box1_base,$
                                 FONT=buttonfont,$
                                 VALUE='Load Spectra',$
                                 UVALUE='Load Spectra')

         box2_base = widget_base(col1_base,$
                                 EVENT_PRO='xcombspec_event',$
                                 /COLUMN,$
                                 /FRAME)
         
            state.w.scaleorder_dl = widget_droplist(box2_base,$
                                                    FONT=buttonfont,$
                                           TITLE='Scale Spectra with Order:',$
                                                    VALUE='None',$
                                                    UVALUE='Scale Order')
            widget_control, state.w.scaleorder_dl, SENSITIVE=0

;            state.w.mask_dl = widget_droplist(box2_base,$
;                                              FONT=buttonfont,$
;                                              TITLE='Make Mask:',$
;                                              VALUE='None',$
;                                              UVALUE='Make Mask')
;            widget_control, state.w.mask_dl, SENSITIVE=0

         box3_base = widget_base(col1_base,$
                                 EVENT_PRO='xcombspec_event',$
                                 /COLUMN,$
                                 /FRAME)

            combine_bg = cw_bgroup(box3_base,$
                                   FONT=buttonfont,$
                                   ['Sum','Weighted Mean','Mean','Median'],$
                                   /ROW,$
                                   /RETURN_NAME,$
                                   /NO_RELEASE,$
                                   /EXCLUSIVE,$
                                   LABEL_TOP='Combination Statistic:',$
                                   UVALUE='Combination Statistic',$
                                   SET_VALUE=3)

            state.w.meanerr_bg = cw_bgroup(box3_base,$
                                           FONT=buttonfont,$
                                           ['RMS','Error on Mean'],$
                                           /ROW,$
                                           /RETURN_INDEX,$
                                           /NO_RELEASE,$
                                           /EXCLUSIVE,$
                                           LABEL_LEFT='Error:',$
                                           UVALUE='Mean Error',$
                                           SET_VALUE=1)
            widget_control, state.w.meanerr_bg, SENSITIVE=0

         box4_base = widget_base(col1_base,$
                                 /COLUMN,$
                                 /BASE_ALIGN_CENTER,$
                                 EVENT_PRO='xcombspec_event',$
                                 /FRAME)

            outformat_bg = cw_bgroup(box4_base,$
                                     FONT=buttonfont,$
                                     ['FITS','Text'],$
                                     /ROW,$
                                     /RETURN_NAME,$
                                     /NONEXCLUSIVE,$
                                     LABEL_LEFT='Output Format:',$
                                     UVALUE='Output Format',$
                                     SET_VALUE=[1,0])
         
            field = coyote_field2(box4_base,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE='Output File:',$
                                  UVALUE='Output File',$
                                  XSIZE=25,$
                                  EVENT_PRO='xcombspec_event',$
                                  /CR_ONLY,$
                                  TEXTID=textid)
            state.w.outfile_fld = [field,textid]

         combine_button = widget_button(box4_base,$
                                        EVENT_PRO='xcombspec_event',$
                                        FONT=buttonfont,$
                                        VALUE='Combine Spectra',$
                                        UVALUE='Combine Spectra')

         state.w.col2_base = widget_base(row_base,$
                                         /COLUMN)

            state.w.aperture_dl = widget_droplist(state.w.col2_base,$
                                                  EVENT_PRO='xcombspec_event',$
                                                  FONT=buttonfont,$
                                                  VALUE='Aperture 1',$
                                                  UVALUE='Aperture')   
            widget_control, state.w.aperture_dl, SENSITIVE=0
            
            state.w.plotwin = widget_draw(state.w.col2_base,$
                                          XSIZE=state.p.plotwinsize[0],$
                                          YSIZE=state.p.plotwinsize[1],$
                                          X_SCROLL_SIZE=state.p.scrollsize[0],$
                                          Y_SCROLL_SIZE=state.p.scrollsize[1],$
                                          UVALUE='Plot Window',$
                                          EVENT_PRO='xcombspec_selectspec')

   button = widget_button(state.w.xcombspec_base,$
                          FONT=buttonfont,$
                          VALUE='Help',$
                          EVENT_PRO='xcombspec_event',$
                          UVALUE='Help')

; Get things running.  Center the widget using the Fanning routine.

centertlb,state.w.xcombspec_base

widget_control, state.w.xcombspec_base, /REALIZE

;  Get plotwin ids

widget_control, state.w.plotwin, GET_VALUE=x
state.p.plotwin_wid = x

window, /FREE, /PIXMAP,XSIZE=state.p.plotwinsize[0],$
  YSIZE=state.p.plotwinsize[1]
state.p.pixmap_wid = !d.window

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xcombspec', $
  state.w.xcombspec_base, $
  /NO_BLOCK,$
  EVENT_HANDLER='xcombspec_resize',$
  CLEANUP='xcombspec_cleanup'

;  Get sizes of things now for resizing

widget_control, state.w.xcombspec_base, TLB_GET_SIZE=result

widget_geom = widget_info(state.w.xcombspec_base, /GEOMETRY)
window_geom = widget_info(state.w.plotwin,  /GEOMETRY)

state.p.buffer[0] = result[0] + 2.*widget_geom.margin - $
  window_geom.xsize
state.p.buffer[1] = result[1] + 2.*widget_geom.margin - $
  window_geom.ysize

; Put state variable into the user value of the top level base.

widget_control, state.w.xcombspec_base, SET_UVALUE=state, /NO_COPY

end






