;+
; NAME:
;     ximgtool
;    
; PURPOSE:
;     General array display widget.
;
; CATEGORY:
;     Widget
;    
; CALLING SEQUENCE:
;     ximgtool
;     ximgtool,img,[hdr],MIN=min,MAX=max,SCALING=scaling,AUTORANGE=autorange,$
;              ROTATION=rotation,GROUP_LEADER=group_leader,WID=wid,$
;              NOUPDATE=noupdate,MASK=mask,STDIMAGE=stdimage,CANCEL=cancel   
; INPUTS:
;    img - 1) A 2D image
;          2) A string array giving the filename of a FITS image
;    
; OPTIONAL INPUTS:
;    hdr - If img is an array, then an optional string array giving
;          the FITS header
;
; KEYWORD PARAMETERS:
;     MIN          - The minimum value to scale the image with
;     MAX          - The maximum value to scale the image with
;     SCALING      - 0 Linear
;                    1 Histrogram Equalization
;
;     AUTORANGE    - 0 Min to Max
;                    1 -5Sigma to +5Sigma
;                    2 0 to Max
;
;     ROTATION     - See IDL user's manual
;     GROUP_LEADER - Used if ximgtool is called from some other
;                    routine
;     WID          - The plot window ID.  Can be used to overplot 
;                    things on the plot window
;     NOUPDATE     - Simply take the input image and plot it using
;                    buffer 1s parameters
;     MASK         - A mask of the same size as img identifying
;                    "bad" pixels.  0=good, 1="bad"
;     CANCEL       - Set on return if there is a problem
;     
; OUTPUTS:
;     None
;     
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     None
;
; PROCEDURE:
;     None
;
; EXAMPLE:
;     
; MODIFICATION HISTORY
;     2002-10-01 - Written by M. Cushing, Institute for Astronomy, UH
;     2003-05-25 - Added MASK keyword
;     2004-09-27 - Added log scaling.
;-
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro ximgtool_startup,STDIMAGE=stdimage,GROUP_LEADER=group_leader


stdimage = (n_elements(stdimage) eq 0) ? 1024:stdimage


common ximgtool_state, state,main_image

;  Load color table

device, RETAIN=2
mkct,BOT=offset,RED=red,GREEN=green,BLUE=blue

;  Build the structures which will hold the important info.
;  w - contains info pertaining to widget operations.

w = {buffer_base:0L,$
     buffer_bg:0L,$
     cbarwin:0L,$
     cenpix_lb:0L,$
     colormap_dl:0L,$
     cp_base:0L,$
     dn_bg:0L,$
     hdr_base:0L,$
     hdr_text:0L,$
     imgmin_fld:[0L,0L],$
     imgmax_fld:[0L,0L],$
     keyboard:0L,$
     plot_base:0,$
     plotwin:0L,$
     saturation_fld:[0L,0L],$
     scaling_bg:0L,$
     track:0L,$
     ximgtool_base:0L}

p = {blue:blue,$
     brightness:500.,$
     buffer:0,$
     winbuffer:[0,0],$
     cbarwin_wid:0L,$
     cenpix:[0.,0.],$
     contrast:500.,$
     currentbuffer:0,$
     cursor:[0.8,0.02],$
     cursormode:'None',$
     dev2img:0.,$
     firstclick:0,$
     green:green,$
     loadedbuffer:[1,0,0,0],$
     maxbandc:1000L,$
     nbuffers:4,$
     ncolors:!d.n_colors<256.0,$
     color_bottom:offset,$
     offset:[0.,0.],$
     pixmap_wid:0L,$
     plotwin_size:[512,512],$
     plotwin_wid:0L,$
     satpixels:0,$
     xscale:!x,$
     yscale:!y,$
     pscale:!p,$
     position:[0.,0.,0.,0.],$
     red:red,$
     reg:[[!values.f_nan,!values.f_nan],$
          [!values.f_nan,!values.f_nan]],$
     stdimage:stdimage,$
     tmpdevpos:[0.,0.],$
     xrange:[0.,0.],$
     yrange:[0.,0.],$
     zoom:0.0}

p.dev2img = p.stdimage/512.

d = {buffer1:{img:ptr_new(2),hdr:ptr_new(''),bytimg:ptr_new(2),$
              min:0.0,max:0.0,ncols:0,nrows:0,cenpix:[0.,0.],zoom:0.0,$
              red:red,green:green,blue:blue,scaling:0,brightness:500,$
              contrast:500,mask:ptr_new(2),saturation:5000.0},$
     buffer2:{img:ptr_new(2),hdr:ptr_new(''),bytimg:ptr_new(2),$
              min:0.0,max:0.0,ncols:0,nrows:0,cenpix:[0.,0.],zoom:0.0,$
              red:red,green:green,blue:blue,scaling:0,brightness:500,$
              contrast:500,mask:ptr_new(2),saturation:5000.0},$
     buffer3:{img:ptr_new(2),hdr:ptr_new(''),bytimg:ptr_new(2),$
              min:0.0,max:0.0,ncols:0,nrows:0,cenpix:[0.,0.],zoom:0.0,$
              red:red,green:green,blue:blue,scaling:0,brightness:500,$
              contrast:500,mask:ptr_new(2),saturation:5000.0},$
     buffer4:{img:ptr_new(2),hdr:ptr_new(''),bytimg:ptr_new(2),$
              min:0.0,max:0.0,ncols:0,nrows:0,cenpix:[0.,0.],zoom:0.0,$
              red:red,green:green,blue:blue,scaling:0,brightness:500,$
              contrast:500,mask:ptr_new(2),saturation:5000.0},$
     bytesclimage:ptr_new(fltarr(2,2)),$
     cbarr:ptr_new(2),$
     dblclick:0.2,$
     dn:0,$
     dns:0,$
     hdr:ptr_new(strarr(2)),$
     mbuf1:0,$
     mbuf2:0,$
     mbuf3:0,$
     mop:0,$
     path:'',$
     plotimg:ptr_new(2),$
     saturation:5000.0}

state = {w:w,p:p,d:d}

;  Make color bar array

b = cmcongrid( findgen(state.p.ncolors-state.p.color_bottom-1), $
               state.p.plotwin_size[0]) + state.p.color_bottom
c = replicate(1,10)
*state.d.cbarr = b # c

;  Get fonts

getfonts,buttonfont,textfont

;  Create the widget

state.w.ximgtool_base = widget_base(TITLE='Ximgtool',$
                                    /COLUMN,$
                                    GROUP_LEADER=group_leader,$
                                    /TLB_SIZE_EVENTS)

   button_row = widget_base(state.w.ximgtool_base,$
                            /ROW,$
                            EVENT_PRO='ximgtool_cp_event',$
                            /BASE_ALIGN_CENTER)

      desc = ['1\File','0\Load FITS','0\Write TIFF','0\View Header','2\Quit']
      pdmenu = cw_pdmenu(button_row,$
                         FONT=buttonfont,$
                         desc,$
                         /RETURN_NAME,$
                         UVALUE='File')   

      desc = ['1\View','1\Range','0\Full Range','0\Auto Range',$
              '2\0-Max','1\Scaling','0\Linear','0\Log',$
              '2\Hist Eq','1\Rotation','0\-Y to +Y (7)','0\-X to +X (5)',$
              '0\-90 (1)','0\-180 (2)','0\-270 (3)','0\(4)','2\(6)',$
              '1\Color','0\Grey',$
              '0\Blue','0\Rainbow','2\Heat']

      pdmenu = cw_pdmenu(button_row,$
                         FONT=buttonfont,$
                         desc,$
                         /RETURN_NAME,$
                         UVALUE='Image View')

      button = widget_button(button_row,$
                             FONT=buttonfont,$
                             VALUE='Panel',$
                             UVALUE='CP')

      button = widget_button(button_row,$
                             FONT=buttonfont,$
                             VALUE='Zoom In',$
                             UVALUE='Zoom In')

      button = widget_button(button_row,$
                             FONT=buttonfont,$
                             VALUE='Zoom Out',$
                             UVALUE='Zoom Out')

      button = widget_button(button_row,$
                             FONT=buttonfont,$
                             VALUE='Center',$
                             UVALUE='Center')

      button = widget_button(button_row,$
                             FONT=buttonfont,$
                             VALUE='Help',$
                             UVALUE='Help')


   state.w.keyboard = widget_text(state.w.ximgtool_base, $
                                  /ALL_EVENTS, $
                                  SCR_XSIZE=1, $
                                  SCR_YSIZE=1, $
                                  UVALUE='Keyboard', $
                                  EVENT_PRO='ximgtool_cp_event',$
                                  VALUE='')

   state.w.track = widget_text(state.w.ximgtool_base, $
                               FONT=textfont,$
                               YSIZE=1)

   plot_base = widget_base(state.w.ximgtool_base,$
                           /COLUMN)
         
      state.w.plotwin = widget_draw(plot_base,$
                                    /ALIGN_CENTER,$
                                    XSIZE=state.p.plotwin_size[0],$
                                    YSIZE=state.p.plotwin_size[1],$
                                    EVENT_PRO='ximgtool_plotwinevent',$
                                    /TRACKING_EVENTS,$
                                    /MOTION_EVENTS,$
                                    /BUTTON_EVENTS)

      state.w.cbarwin = widget_draw(plot_base,$
                                    /ALIGN_CENTER,$
                                    YSIZE=10,$
                                    XSIZE=state.p.plotwin_size[0])
      

; Get things running.

centertlb,state.w.ximgtool_base
      
widget_control, state.w.ximgtool_base, /REALIZE

;  Get plotwin ids

widget_control, state.w.plotwin, GET_VALUE=x
state.p.plotwin_wid=x
wid=x

widget_control, state.w.cbarwin, GET_VALUE=x
state.p.cbarwin_wid=x

window, /FREE, /PIXMAP,XSIZE=state.p.plotwin_size[0],$
  YSIZE=state.p.plotwin_size[1]
state.p.pixmap_wid=!d.window

;  Get sizes for things.

widget_geom = widget_info(state.w.ximgtool_base, /GEOMETRY)

state.p.winbuffer[0]=widget_geom.xsize-state.p.plotwin_size[0]
state.p.winbuffer[1]=widget_geom.ysize-state.p.plotwin_size[1]


; Start the Event Loop. This will be a non-blocking program.

XManager, 'ximgtool', $
  state.w.ximgtool_base, $
  EVENT_HANDLER='ximgtool_resize',$
  /NO_BLOCK,$
  CLEANUP='ximgtool_cleanup'

end
;
;******************************************************************************
;
pro ximgtool_bgupdate

common ximgtool_state

widget_control, state.w.buffer_base,UPDATE=0
widget_control, state.w.buffer_bg,/DESTROY

z      = where(state.p.loadedbuffer eq 1,count)
values = string(z+1,FORMAT='(i2.2)')

getfonts,buttonfont,textfont

state.w.buffer_bg = cw_bgroup(state.w.buffer_base,$
                              FONT=buttonfont,$
                              values,$
                              /ROW,$
                              /RETURN_NAME,$
                              /NO_RELEASE,$
                              /EXCLUSIVE,$
                              LABEL_LEFT='Display Buffer:',$
                              UVALUE='Display Buffer',$
                              SET_VALUE=state.p.buffer)

widget_control, state.w.buffer_base,UPDATE=1

end
;
;******************************************************************************
;
pro ximgtool_bytsclimg


common ximgtool_state

ncolors = state.p.ncolors-(state.p.color_bottom+1)

case state.d.(state.p.buffer).scaling of 

    0: img = bytscl(*state.d.(state.p.buffer).img,/NAN,$
                    MIN=state.d.(state.p.buffer).min,$
                    MAX=state.d.(state.p.buffer).max,$
                    TOP=ncolors) + state.p.color_bottom

    1: begin

;  Ripped from ATV

        offset = state.d.(state.p.buffer).min - $
          (state.d.(state.p.buffer).max - state.d.(state.p.buffer).min)*0.01

        img = bytscl(alog10(*state.d.(state.p.buffer).img-offset), $
                     MIN=alog10(state.d.(state.p.buffer).min-offset),$
                     MAX=alog10(state.d.(state.p.buffer).max-offset),$
                     /NAN,TOP=ncolors)+state.p.color_bottom


    end
    2: img = bytscl(hist_equal(*state.d.(state.p.buffer).img, $
                               MINV=state.d.(state.p.buffer).min,$
                               MAXV=state.d.(state.p.buffer).max),$
                    /NAN,TOP=ncolors)+state.p.color_bottom



endcase



z = where(*state.d.(state.p.buffer).mask eq 1, count)
if count ne 0 then img[z] = 2.0
    
*state.d.(state.p.buffer).bytimg = img

end
;
;******************************************************************************
;
pro ximgtool_cleanup,event

common ximgtool_state

ptr_free, state.d.(0).img
ptr_free, state.d.(1).img
ptr_free, state.d.(2).img
ptr_free, state.d.(3).img

ptr_free, state.d.(0).bytimg
ptr_free, state.d.(1).bytimg
ptr_free, state.d.(2).bytimg
ptr_free, state.d.(3).bytimg

ptr_free, state.d.(0).hdr
ptr_free, state.d.(1).hdr
ptr_free, state.d.(2).hdr
ptr_free, state.d.(3).hdr


state = 0B

end
;
;*****************************************************************************
;
pro ximgtool_color,event,BRIGHTNESS=brightness,CONTRAST=contrast

;  Ripped directly from ATV

common ximgtool_state

pwin = state.p.plotwin_size

if n_elements(BRIGHTNESS) eq 0 then begin

    x = 1 > event.x < pwin[0]
    y = 1 > event.y < pwin[1]

    brightness = long ( (pwin[1]-y) * (state.p.maxbandc)/pwin[1])
    contrast   = long ( x * (state.p.maxbandc)/pwin[0])

    state.d.(state.p.buffer).brightness = brightness
    state.d.(state.p.buffer).contrast   = contrast

endif 

d = state.p.ncolors-state.p.color_bottom-1

maxdp = 600
mindp = 4

if (contrast LT (state.p.maxbandc / 2)) then begin
    dp = ((d - maxdp) / float((state.p.maxbandc / 2) - 1)) * contrast + $
      ((maxdp * state.p.maxbandc / 2) - d) / float(state.p.maxbandc / 2)
endif else begin
    dp = ((mindp - d) / float(state.p.maxbandc / 2)) * contrast + $
      ((d * state.p.maxbandc) - (mindp * state.p.maxbandc / 2)) / $
      float(state.p.maxbandc / 2)
endelse

dp =  fix(dp)

r = replicate( (state.d.(state.p.buffer).red)[d-1]  , 2*d + dp)
g = replicate( (state.d.(state.p.buffer).green)[d-1], 2*d + dp)
b = replicate( (state.d.(state.p.buffer).blue)[d-1] , 2*d + dp)

r[0:d-1] = (state.d.(state.p.buffer).red)[0]
g[0:d-1] = (state.d.(state.p.buffer).green)[0]
b[0:d-1] = (state.d.(state.p.buffer).blue)[0]

a = findgen(d)

r[d] = congrid(state.d.(state.p.buffer).red, dp)
g[d] = congrid(state.d.(state.p.buffer).green, dp)
b[d] = congrid(state.d.(state.p.buffer).blue, dp)

bshift = round(brightness * (d+dp) / float(state.p.maxbandc))

rr = r[a + bshift] 
gg = g[a + bshift]
bb = b[a + bshift]

tvlct, rr, gg, bb, state.p.color_bottom

end
;
;******************************************************************************
;
pro ximgtool_hdr

common ximgtool_state

if not xregistered('ximgtool_hdrinfo') then begin

    getfonts,buttonfont,textfont

    hdr_base = widget_base(GROUP_LEADER=state.w.ximgtool_base, $
                           /COLUMN, $
                           TITLE='FITS Header')

       hdr_text = widget_text(hdr_base,$
                              /SCROLL,$
                              FONT=textfont,$
                              VALUE=*state.d.(0).hdr, $
                              XSIZE=85, $
                              YSIZE=24)
       
       button = widget_button(hdr_base,$
                              FONT=buttonfont,$
                              EVENT_PRO='ximgtool_cp_event',$
                              VALUE='Done',$
                              UVALUE='Done')
    
    widget_control, hdr_base, /REALIZE

    XManager, 'hdrinfo', $
      hdr_base, $
      /NO_BLOCK
    
endif

end
;
;******************************************************************************
;
pro ximgtool_help

common ximgtool_state

if not xregistered('ximgtool_help') then begin

    getfonts,buttonfont,textfont

    arr = [['---Ximgtool----'],$
           ['Ximgtool is resizable widget that can be used to view FITS images.'],$        ['It can also be used to view any two-dimensional array.'],$
           [''],$
           ['It can be called from the command line as:'],$
           [''],$
           ['IDL> ximgtool'],$
           ['IDL> ximgtool,img,[hdr],MIN=min,MAX=max,SCALING=scaling,$'],$
           ['              AUTORANGE=autorange,ROTATION=rotation,$'],$
           ['              GROUP_LEADER=group_leader,WID=wid,NOUPDATE=noupdate,$'],$
           ['              CANCEL=cancel'],$
           ['where:'],$
           [''],$
           ['  img - 1) A 2D image'],$
           ['        2) A string array giving the filename of a FITS image'],$
           ['  hdr - If img is an array, then an optional string array giving the FITS header.'],$
           [''],$
           ['MIN          - The minimum value to scale the image with'],$
           ['MAX          - The maximum value to scale the image with'],$
           ['SCALING      - 0 Linear'],$
           ['               1 Histrogram Equalization'],$
           ['AUTORANGE    - 0 Min to Max'],$
           ['               1 -5Sigma to +5Sigma'],$
           ['               2 0 to Max'],$
           ['ROTATION     - See IDL manual'],$
           ['GROUP_LEADER - Used if ximgtool is called from some other routine'],$
           ['WID          - The plot window ID.  Can be used to overplot things on the plot window'],$
           ['NOUPDATE     - Simply take the input image and plot it using buffer 1s parameters'],$
           ['CANCEL       - Set on return if there is a problem'],$
           [''],$
           ['Zoom Options:'],$
           [''],$
           ['  Click the mouse once to center the image.'],$
           ['  Double click the mouse to zoom in and center.']]

    hdr_base = widget_base(GROUP_LEADER=state.w.ximgtool_base, $
                           /COLUMN, $
                           TITLE='Ximtool Help')

       hdr_text = widget_text(hdr_base,$
                              /SCROLL,$
                              FONT=textfont,$
                              VALUE=arr, $
                              XSIZE=85, $
                              YSIZE=24)
       
       button = widget_button(hdr_base,$
                              FONT=buttonfont,$
                              EVENT_PRO='ximgtool_cp_event',$
                              VALUE='Done',$
                              UVALUE='Done')
    
    widget_control, hdr_base, /REALIZE

    XManager, 'help', $
      hdr_base, $
      /NO_BLOCK
    
endif

end
;
;******************************************************************************
; 
pro ximgtool_imgrange,index

common ximgtool_state

case index of 

    0: min = min(*state.d.(state.p.buffer).img,/NAN,MAX=max)

    1: begin

        robuststats,*state.d.(state.p.buffer).img,10,mean,dummy,stddev,/SILENT
        min = (mean-3.*stddev) > min(*state.d.(state.p.buffer).img,/NAN,MA=max)
        max = (mean+7.*stddev) < max

    end

    2: begin

        min = 0
        max = max(*state.d.(state.p.buffer).img,/NAN)

    end

endcase

state.d.(state.p.buffer).min = min
state.d.(state.p.buffer).max = max

if xregistered('ximgtool_cp') then begin

    widget_control, state.w.imgmin_fld[1],SET_VALUE=strtrim(min,2)
    widget_control, state.w.imgmax_fld[1],SET_VALUE=strtrim(max,2)

endif

end
;
;******************************************************************************
;
pro ximgtool_linecut

common ximgtool_state

!x = state.p.xscale
!y = state.p.yscale
!p = state.p.pscale

p1 = convert_coord(state.p.reg[*,0],/DEVICE,/TO_DATA)
p2 = convert_coord(state.p.reg[*,1],/DEVICE,/TO_DATA)

nPoints = ABS(p2[0]-p1[0]+1) > ABS(p2[1]-p1[1]+1)

xloc = p1[0] + (p2[0] - p1[0]) * Findgen(nPoints) / (nPoints - 1)
yloc = p1[1] + (p2[1] - p1[1]) * Findgen(nPoints) / (nPoints - 1)

z = where(xloc gt 0 and yloc gt 0 and xloc lt state.d.(state.p.buffer).ncols $
          and yloc lt state.d.(state.p.buffer).nrows)


profile = Interpolate(*state.d.(state.p.buffer).img, xloc, yloc)

xzoomplot,findgen(n_elements(profile)),float(profile)

state.p.reg[*] = !values.f_nan


end
;
;******************************************************************************
;
pro ximgtool_minmax

common ximgtool_state

min = cfld(state.w.imgmin_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
max = cfld(state.w.imgmax_fld,4,/EMPTY,CANCEL=cancel)
if cancel then return
if min ge max then begin

    widget_control, state.w.imgmin_fld[0], $
      SET_VALUE=strtrim(state.d.(state.p.buffer).min,2)
    widget_control, state.w.imgmax_fld[0], $
      SET_VALUE=strtrim(state.d.(state.p.buffer).max,2)

endif else begin

    state.d.(state.p.buffer).min = min
    state.d.(state.p.buffer).max = max
    ximgtool_bytsclimg
    ximgtool_mkplotimg
    ximgtool_plotupdate
    
endelse

end
;
;******************************************************************************
;
pro ximgtool_mkcp

common ximgtool_state, state

if xregistered('ximgtool_cp') then goto, getout

getfonts,buttonfont,textfont

buffers = string(findgen(state.p.nbuffers)+1,FORMAT='(i2.2)')
state.w.cp_base = widget_base(TITLE='Ximgtool-Control Panel',$
                              GROUP_LEADER=state.w.ximgtool_base,$
                              /COLUMN)


   button = widget_button(state.w.cp_base,$
                          FONT=buttonfont,$
                          VALUE='Done',$
                          UVALUE='Done')


   row_base = widget_base(state.w.cp_base,$
                          /ROW)

      image_base = widget_base(row_base,$
                               /COLUMN,$
                               /FRAME)

         state.w.buffer_base = widget_base(image_base,$
                                           /COLUMN)

            z      = where(state.p.loadedbuffer eq 1,count)
            values = string(z+1,FORMAT='(i2.2)')
            
            state.w.buffer_bg = cw_bgroup(state.w.buffer_base,$
                                          FONT=buttonfont,$
                                          values,$
                                          /ROW,$
                                          /RETURN_NAME,$
                                          /NO_RELEASE,$
                                          /EXCLUSIVE,$
                                          LABEL_LEFT='Display Buffer:',$
                                          UVALUE='Display Buffer',$
                                          SET_VALUE=state.p.buffer)
            widget_control, state.w.buffer_bg, SET_VALUE=state.p.buffer
            
            
            dl = widget_droplist(image_base,$
                                 FONT=buttonfont,$
                                 TITLE='Load Buffer:',$
                                 VALUE=buffers,$
                                 UVALUE='Load Buffer')

            irtf_base = widget_base(image_base,$
                                    /ROW)            

               state.w.dn_bg = cw_bgroup(irtf_base,$
                                         FONT=buttonfont,$
                                         ['DN'],$
                                         /ROW,$
                                         /RETURN_NAME,$
                                         /NONEXCLUSIVE,$
                                         UVALUE='DN',$
                                         SET_VALUE=state.d.dn)
               

               bg = cw_bgroup(irtf_base,$
                              FONT=buttonfont,$
                              ['Saturation'],$
                              /ROW,$
                              /RETURN_NAME,$
                              /NONEXCLUSIVE,$
                              UVALUE='Plot Saturation',$
                              SET_VALUE=state.p.satpixels)

               fld = coyote_field2(irtf_base,$
                                   LABELFONT=buttonfont,$
                                   FIELDFONT=textfont,$
                                   TITLE=':',$
                                   VALUE=5000,$
                                   UVALUE='Saturation Level',$
                                   EVENT_PRO='ximgtool_cp_event',$
                                   /CR_ONLY,$
                                   XSIZE=6,$
                                   TEXTID=textid)
            state.w.saturation_fld = [fld,textid]   
         

         zoom_base = widget_base(image_base,$
                                 /ROW)

            button = widget_button(zoom_base,$
                                   FONT=buttonfont,$
                                   VALUE='Zoom In',$
                                   UVALUE='Zoom In')
            
            button = widget_button(zoom_base,$
                                   FONT=buttonfont,$
                                   VALUE='Zoom Out',$
                                   UVALUE='Zoom Out')
            
            button = widget_button(zoom_base,$
                                   FONT=buttonfont,$
                                   VALUE='Zoom 1',$
                                   UVALUE='Zoom 1')
            
         center_base = widget_base(image_base,$
                                   /ROW)

            button = widget_button(center_base,$
                                   FONT=buttonfont,$
                                   VALUE='Center',$
                                   UVALUE='Center')


            string = 'Center Pix:  '+$
              strjoin(strtrim(state.d.(state.p.buffer).cenpix,2),',  ')
            state.w.cenpix_lb = widget_label(center_base,$
                                             FONT=buttonfont,$
                                             VALUE=string)

      mod_base = widget_base(row_base,$
                             /COLUMN,$
                             /FRAME)

         desc = ['1\Range','0\Full Range','0\Auto Range','2\0-Max']
         pdmenu = cw_pdmenu(mod_base,$
                            FONT=buttonfont,$
                            desc,$
                            /RETURN_INDEX,$
                            UVALUE='Image Range')
      
         desc = ['1\Image Scaling','0\Linear','2\Hist Eq']
         pdmenu = cw_pdmenu(mod_base,$
                            FONT=buttonfont,$
                            desc,$
                            /RETURN_INDEX,$
                            UVALUE='Image Scaling')
         
         desc = ['1\Rotation','0\-Y to +Y (7)','0\-X to +X (5)','0\-90 (1)',$
                 '0\-180 (2)','0\-270 (3)','0\(4)','2\(6)']
         pdmenu = cw_pdmenu(mod_base,$
                            FONT=buttonfont,$
                            desc,$
                            /RETURN_INDEX,$
                            UVALUE='Rotation')
         
         desc = ['1\Color Map','0\Grey','0\Blue','0\Rainbow','2\Heat']
         pdmenu = cw_pdmenu(mod_base,$
                            FONT=buttonfont,$
                            desc,$
                            /RETURN_INDEX,$
                            UVALUE='Color Map')
         
         fld = coyote_field2(mod_base,$
                             LABELFONT=buttonfont,$
                             FIELDFONT=textfont,$
                             TITLE='Image Min:',$
                             VALUE=strtrim(state.d.(state.p.buffer).min,2),$
                             UVALUE='Image Min',$
                             EVENT_PRO='ximgtool_cp_event',$
                             XSIZE=15,$
                             /CR_ONLY,$
                             TEXTID=textid)
         state.w.imgmin_fld = [fld,textid]   
         
         fld = coyote_field2(mod_base,$
                             LABELFONT=buttonfont,$
                             FIELDFONT=textfont,$
                             TITLE='Image Max:',$
                             VALUE=strtrim(state.d.(state.p.buffer).max,2),$
                             UVALUE='Image Max',$
                             EVENT_PRO='ximgtool_cp_event',$
                             XSIZE=15,$
                             /CR_ONLY,$
                             TEXTID=textid)
         state.w.imgmax_fld = [fld,textid]   
   

   math_base = widget_base(state.w.cp_base,$
                           /ROW,$
                           /FRAME,$
                           /BASE_ALIGN_CENTER)

      dl = widget_droplist(math_base,$
                           FONT=buttonfont,$
                           TITLE='Buffer Math:',$
                           VALUE=buffers,$
                           UVALUE='Math Buffer 1')

     dl = widget_droplist(math_base,$
                           FONT=buttonfont,$
                           VALUE=['+','-','*','/'],$
                           UVALUE='Math Operation')


     dl = widget_droplist(math_base,$
                           FONT=buttonfont,$
                           VALUE=buffers,$
                           UVALUE='Math Buffer 2')

     label = widget_label(math_base,$
                          FONT=buttonfont,$
                          VALUE='=')

     dl = widget_droplist(math_base,$
                           FONT=buttonfont,$
                           VALUE=buffers,$
                           UVALUE='Math Buffer 3')

     button = widget_button(math_base,$
                             FONT=buttonfont,$
                             VALUE='Compute',$
                             UVALUE='Compute Math')

; Get things running.

centertlb,state.w.cp_base
      
widget_control, state.w.cp_base, /REALIZE


; Start the Event Loop. This will be a non-blocking program.

XManager, 'ximgtool_cp', $
  state.w.cp_base, $
  /No_Block


getout:

end
;
;*****************************************************************************
;
pro ximgtool_mkplotimg

common ximgtool_state

;  Get zoom window size.

zoom_factor = 2.^state.d.(state.p.buffer).zoom
zwin_size   = state.p.plotwin_size / zoom_factor * state.p.dev2img
dev_offset  = (zwin_size-floor(zwin_size)) * zoom_factor / state.p.dev2img / 2.
zwin_size   = floor(zwin_size)
pwin_size   = round(state.p.plotwin_size-dev_offset)

;  Get limits of image for zoom window.

xyll        = roundgt(state.d.(state.p.buffer).cenpix - zwin_size/2. ) 
xyur        = xyll+zwin_size-1
xyll_img    = xyll > 0.
xyur_img    = fltarr(2)
xyur_img[0] = xyur[0] < (state.d.(state.p.buffer).ncols-1)
xyur_img[1] = xyur[1] < (state.d.(state.p.buffer).nrows-1)

zwin_imgsize = xyur_img-xyll_img+1
zwin_offset  = xyll_img-xyll

pwin_imgsize = round( (pwin_size/float(zwin_size))*zwin_imgsize)
pwin_offset  = round( (pwin_size/float(zwin_size))*zwin_offset)

img = *state.d.(state.p.buffer).bytimg

*state.d.plotimg = cmcongrid(img[xyll_img[0]:xyur_img[0],$
                                 xyll_img[1]:xyur_img[1]],$
                             pwin_imgsize[0],pwin_imgsize[1])

state.p.offset = pwin_offset+dev_offset

state.p.xrange   = xyll_img[0]+[0,zwin_imgsize[0]]-0.5
state.p.yrange   = xyll_img[1]+[0,zwin_imgsize[1]]-0.5
state.p.position = [state.p.offset[0],state.p.offset[1],$
                    state.p.offset[0]+pwin_imgsize[0],$
                    state.p.offset[1]+pwin_imgsize[1]]

end
;
;******************************************************************************
;
pro ximgtool_plotupdate,ERASE=erase,CENTER=center

common ximgtool_state

wset, state.p.cbarwin_wid
tv,*state.d.cbarr

wset, state.p.pixmap_wid
if keyword_set(ERASE) then erase
tv, *state.d.plotimg,state.p.offset[0],state.p.offset[1],/DEVICE

;  Set to one for debugging

style = 5

plot,indgen(10),/NODATA,/NOERASE,XSTY=style,YSTY=style,XRANGE=state.p.xrange,$
  YRANGE=state.p.yrange,XMARGIN=[0,0],YMARGIN=[0,0],COLOR=2,NOCLIP=0,$
  /DEVICE,POSITION=state.p.position

state.p.xscale = !x
state.p.yscale = !y
state.p.pscale = !p

wset, state.p.plotwin_wid
device, copy=[0,0,state.p.plotwin_size[0],state.p.plotwin_size[1],0,0,$
              state.p.pixmap_wid]

if keyword_set(CENTER) then tvcrs,roundgt(state.d.(state.p.buffer).cenpix[0]),$
  roundgt(state.d.(state.p.buffer).cenpix[1]),/DATA


end
;
;******************************************************************************
;
pro ximgtool_writetiff

;  Taken from A. Barth's atv 

common ximgtool_state

filename = dialog_pickfile(DIALOG_PARENT=state.w.ximgtool_base,$
                           FILTER='*.tiff',/WRITE,$
                           FILE='img.tiff')

if filename ne '' then  begin

    tmp_img = tvrd(/TRUE)
    tmp_img = reverse(tmp_img, 3)
    write_tiff, filename, tmp_img, 1, /planarconfig

endif
end
;
;******************************************************************************
;
; ----------------------------Event Procedures------------------------------ 
;
;******************************************************************************
;
pro ximgtool_cp_event,event

common ximgtool_state

widget_control, event.id, GET_UVALUE=uvalue
widget_control, /HOURGLASS

case uvalue of

    'Center': begin

        state.d.(state.p.buffer).cenpix = [state.d.(state.p.buffer).ncols,$
                                           state.d.(state.p.buffer).nrows]/2.-$
          0.5
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE

        if xregistered('ximgtool_cp') then begin
            
            string='Center Pix:  '+$
              strjoin(strtrim(state.d.(state.p.buffer).cenpix,2),',  ')
            widget_control, state.w.cenpix_lb,SET_VALUE=string

        endif

    end

    'Color Map': begin

        mkct,event.value-1,RED=red,GREEN=green,BLUE=blue
        state.d.(state.p.buffer).red   = red
        state.d.(state.p.buffer).green = green
        state.d.(state.p.buffer).blue  = blue
        ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
          CONTRAST=state.d.(state.p.buffer).contrast
        ximgtool_plotupdate

    end

    'Compute Math': begin

        case state.d.mop of 

            0: begin

                tmp=*state.d.(state.d.mbuf1).img + *state.d.(state.d.mbuf2).img
                mask = *state.d.(state.d.mbuf1).mask > $
                  *state.d.(state.d.mbuf2).mask

            end

            1: begin

                tmp=*state.d.(state.d.mbuf1).img - *state.d.(state.d.mbuf2).img
                mask = *state.d.(state.d.mbuf1).mask > $
                  *state.d.(state.d.mbuf2).mask

            end

            2: begin

                tmp=*state.d.(state.d.mbuf1).img * $
                  float(*state.d.(state.d.mbuf2).img)
                mask = *state.d.(state.d.mbuf1).mask > $
                  *state.d.(state.d.mbuf2).mask

            end

            3: begin
                
                tmp=*state.d.(state.d.mbuf1).img / $
                  float(*state.d.(state.d.mbuf2).img)
                mask = *state.d.(state.d.mbuf1).mask > $
                  *state.d.(state.d.mbuf2).mask

            end
        end

        *state.d.(state.d.mbuf3).img  = tmp
        *state.d.(state.d.mbuf3).mask = mask
                                               
        
        state.d.(state.d.mbuf3).ncols = state.d.(state.d.mbuf1).ncols
        state.d.(state.d.mbuf3).nrows = state.d.(state.d.mbuf1).nrows
        
        state.d.(state.d.mbuf3).cenpix =[state.d.(state.d.mbuf1).ncols,$
                                         state.d.(state.d.mbuf1).nrows]/2.-0.5
        state.d.(state.d.mbuf3).zoom = 0.0
        state.d.(state.d.mbuf3).min = min(*state.d.(state.d.mbuf3).img,$
                                        /NAN,MAX=max)
        state.d.(state.d.mbuf3).max = max
        
        state.d.(state.d.mbuf3).scaling = 0
        
;  Update buffer display bg
        
        state.p.loadedbuffer[state.d.mbuf3] = 1
        ximgtool_bgupdate

    end

    'Display Buffer': begin

;  Load new color map

        state.p.buffer = fix(event.value)-1

        tvlct,state.d.(state.p.buffer).red,state.d.(state.p.buffer).green,$
          state.d.(state.p.buffer).blue,state.p.color_bottom

        ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
          CONTRAST=state.d.(state.p.buffer).contrast

        ximgtool_bytsclimg
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE

        widget_control, state.w.imgmin_fld[1],$
          SET_VALUE=strtrim(state.d.(state.p.buffer).min,2)
        widget_control, state.w.imgmax_fld[1],$
          SET_VALUE=strtrim(state.d.(state.p.buffer).max,2)
        widget_control, state.w.saturation_fld[1],$
          SET_VALUE=strtrim(state.d.(state.p.buffer).saturation,2)

        
    end

    'DN': begin

        state.d.dn = event.select
        if state.d.dn eq 1 and state.d.dns eq 1 then begin

            state.d.dns = 0
            widget_control,state.w.dns_bg, SET_VALUE=0 

        endif

    end

    'Done': widget_control, event.top, /DESTROY

    'File': begin

        case strtrim(event.value,2) of 

            'Load FITS': begin

                fullpath=dialog_pickfile(DIALOG_PARENT=state.w.ximgtool_base,$
                                         FILTER=['*.fits','*gz'],$
                                         GET_PATH=newpath,$
                                         /MUST_EXIST,PATH=state.d.path)
                state.d.path =newpath
                if fullpath ne '' then begin
                    
                    img = readfits(fullpath,hdr)
                    s = size(img)
                    
                    if s[0] ne 2 then begin
                        
                        result = dialog_message('FITS array must bd 2D.',$
                                                /ERROR,DIALOG_PARENT=$
                                                state.w.ximgtool_base)
                        return

                    endif

;  Load image
                    
                    *state.d.(0).img  = readfits(fullpath,hdr)
                    *state.d.(0).hdr  = hdr

                    if state.d.dn then begin

                        d = fxpar(hdr,'DIVISOR')
                        *state.d.(0).img = *state.d.(0).img/float(d)

                    endif

                    state.d.(0).ncols = s[1]
                    state.d.(0).nrows = s[2]
                    
                    state.d.(0).cenpix  = [state.d.(0).ncols,$
                                           state.d.(0).nrows]/2.-0.5
                    state.d.(0).zoom    = 0.0
                    state.d.(0).min     = min(*state.d.(0).img,/NAN,MAX=max)
                    state.d.(0).max     = max
                    
                    state.d.(0).scaling = 0
                    state.p.loadedbuffer[0] = 1                    
                    
;  Make mask

                    *state.d.(0).mask = intarr(s[1],s[2])

                    if state.p.satpixels then begin

                        z = where(*state.d.(0).img gt $
                                  state.d.(0).saturation,cnt)                  
                        if cnt gt 0 then (*state.d.(0).mask)[z] = 1.0
                        
                    endif
                    if xregistered('ximgtool_cp') then begin
                        
                        ximgtool_bgupdate
                        
;  Check to see if the current buffer has been modified.
                        
                        if state.p.buffer eq 0 then begin
                            
                            widget_control, state.w.imgmin_fld[1],$
                              SET_VALUE=strtrim(state.d.(0).min,2)
                            widget_control, state.w.imgmax_fld[1],$
                              SET_VALUE=strtrim(state.d.(0).max,2)
                           
                        endif
                        
                    endif
                    
                    ximgtool_imgrange,2
                    ximgtool_bytsclimg
                    ximgtool_mkplotimg
                    ximgtool_plotupdate,/ERASE
                    
                endif
                                
            end

            'View Header': ximgtool_hdr

            'Write TIFF': ximgtool_writetiff

            'Quit': widget_control, event.top, /DESTROY

        endcase

    end

    'Help': ximgtool_help

    'Image Min': ximgtool_minmax

    'Image Max': ximgtool_minmax

    'Image Range': begin

        ximgtool_imgrange,event.value-1
        ximgtool_bytsclimg
        ximgtool_mkplotimg
        ximgtool_plotupdate

    end

    'Image Scaling': begin

        state.d.(state.p.buffer).scaling = event.value-1
        ximgtool_bytsclimg
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE 

    end

    'Image View': begin

        case strtrim(event.value,2) of 

            'Blue': begin

                mkct,1,RED=red,GREEN=green,BLUE=blue
                state.d.(state.p.buffer).red   = red
                state.d.(state.p.buffer).green = green
                state.d.(state.p.buffer).blue  = blue
                state.p.green = green
                state.p.red = red
                state.p.blue = blue
                ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
                  CONTRAST=state.d.(state.p.buffer).contrast
                ximgtool_plotupdate

            end

            'Full Range': begin

                ximgtool_imgrange,0
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate

            end

            'Grey': begin

                mkct,0,RED=red,GREEN=green,BLUE=blue
                state.d.(state.p.buffer).red   = red
                state.d.(state.p.buffer).green = green
                state.d.(state.p.buffer).blue  = blue
                state.p.green = green
                state.p.red = red
                state.p.blue = blue
                ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
                  CONTRAST=state.d.(state.p.buffer).contrast
                ximgtool_plotupdate

            end

            'Heat': begin

                mkct,3,RED=red,GREEN=green,BLUE=blue
                state.d.(state.p.buffer).red   = red
                state.d.(state.p.buffer).green = green
                state.d.(state.p.buffer).blue  = blue
                state.p.green = green
                state.p.red = red
                state.p.blue = blue
                ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
                  CONTRAST=state.d.(state.p.buffer).contrast
                ximgtool_plotupdate

            end

            'Hist Eq':  begin

                state.d.(state.p.buffer).scaling = 2
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE    

            end

            '0-Max': begin

                ximgtool_imgrange,2
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate

            end
            'Auto Range': begin

                ximgtool_imgrange,1
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate

            end

            'Linear': begin

                state.d.(state.p.buffer).scaling = 0
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE    

            end

            'Log':  begin

                state.d.(state.p.buffer).scaling = 1
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE    

            end

            'Rainbow': begin

                mkct,2,RED=red,GREEN=green,BLUE=blue
                state.d.(state.p.buffer).red   = red
                state.d.(state.p.buffer).green = green
                state.d.(state.p.buffer).blue  = blue
                state.p.green = green
                state.p.red = red
                state.p.blue = blue
                ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
                  CONTRAST=state.d.(state.p.buffer).contrast
                ximgtool_plotupdate

            end

            '-Y to +Y (7)': begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,7)
                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,7)

                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5

                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE


            end

            '-X to +X (5)': begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,5)

                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,5)


                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5

       
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

            end

            '-90 (1)':begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,1)

                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,1)


                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5
       
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

            end

            '-180 (2)': begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,2)

                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,2)



                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5
       
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

                
            end

            '-270 (3)': begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,3)

                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,3)


                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5
      
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

            end

         '(4)':begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,4)

                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,4)


                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5
       
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

            end

         '(6)':begin

                *state.d.(state.p.buffer).img = $
                  rotate(*state.d.(state.p.buffer).img,4)

                *state.d.(state.p.buffer).mask = $
                  rotate(*state.d.(state.p.buffer).mask,4)


                s = size(*state.d.(state.p.buffer).img)
                state.d.(state.p.buffer).ncols = s[1]
                state.d.(state.p.buffer).nrows = s[2]
                
                state.d.(state.p.buffer).cenpix = $
                  [state.d.(state.p.buffer).ncols,$
                   state.d.(state.p.buffer).nrows]/2.-0.5
       
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

            end

            else:
            
        endcase

    end

    'Load Buffer': begin
        
        fullpath = dialog_pickfile(DIALOG_PARENT=state.w.ximgtool_base,$
                                   FILTER=['*.fits','*gz'],GET_PATH=newpath,$
                                   /MUST_EXIST,PATH=state.d.path)
        state.d.path =newpath
        if fullpath ne '' then begin
            
            img = readfits(fullpath,hdr)
            s   = size(img)
            
            if s[0] ne 2 then begin
                
                result = dialog_message('FITS array must bd 2D.',$
                                        /ERROR,$
                                        DIALOG_PARENT=state.w.ximgtool_base)
                return

            endif

;  Load image

            *state.d.(event.index).img = img
            *state.d.(event.index).hdr = hdr

            if state.d.dn then begin
                
                d = fxpar(hdr,'DIVISOR')
                *state.d.(event.index).img = $
                  *state.d.(event.index).img/float(d)
                
            endif
            
            state.d.(event.index).ncols = s[1]
            state.d.(event.index).nrows = s[2]

            state.d.(event.index).cenpix  = [state.d.(event.index).ncols,$
                                             state.d.(event.index).nrows]/2.$
            -0.5
            state.d.(event.index).zoom = 0.0
            state.d.(event.index).min = min(*state.d.(event.index).img,$
                                            /NAN,MAX=max)
            state.d.(event.index).max = max

            state.d.(event.index).scaling = 0

            *state.d.(event.index).mask = intarr(s[1],s[2])

            if state.p.satpixels then begin

                z = where(*state.d.(event.index).img gt $
                          state.d.(event.index).saturation,cnt)
                if cnt gt 0 then (*state.d.(event.index).mask)[z] = 1.0
            
            endif 

;  Update buffer display bg
            
            state.p.loadedbuffer[event.index] = 1
            ximgtool_bgupdate

            mkct,0,RED=red,GREEN=green,BLUE=blue
            state.d.(event.index).red   = red
            state.d.(event.index).green = green
            state.d.(event.index).blue  = blue
            
;  Check to see if the current buffer has been modified.

            if state.p.buffer eq event.index then begin

                widget_control, state.w.imgmin_fld[1],$
                  SET_VALUE=strtrim(state.d.(event.index).min,2)
                widget_control, state.w.imgmax_fld[1],$
                  SET_VALUE=strtrim(state.d.(event.index).max,2)

                state.p.green = green
                state.p.red   = red
                state.p.blue  = blue

                ximgtool_imgrange,2
                ximgtool_bytsclimg
                ximgtool_mkplotimg
                ximgtool_plotupdate,/ERASE

            endif else begin

                state.p.red   = state.d.(state.p.buffer).red                
                state.p.green = state.d.(state.p.buffer).green
                state.p.blue  = state.d.(state.p.buffer).blue
                tvlct,state.p.red,state.p.green,state.p.blue,$
                  state.p.color_bottom

            endelse

        endif

    end

    'Keyboard': begin

        case strtrim(event.ch,2) of 

            'l': begin

                state.p.cursormode='Line Cut'
                state.p.reg = !values.f_nan
                ximgtool_plotupdate,/ERASE
            
            end

            'c': begin

                state.p.cursormode='None'
                state.p.reg = !values.f_nan
                ximgtool_plotupdate,/ERASE

            end

            else:

        endcase
    end

    'Math Buffer 1': state.d.mbuf1 = event.index

    'Math Buffer 2': state.d.mbuf2 = event.index

    'Math Buffer 3': state.d.mbuf3 = event.index

    'Math Operation': state.d.mop = event.index

    'Plot Saturation': begin

        state.p.satpixels = event.select
        if state.p.satpixels then begin
            
            *state.d.(state.p.buffer).mask = $
              intarr(state.d.(state.p.buffer).ncols,$
                     state.d.(state.p.buffer).nrows)
            
            z = where(*state.d.(state.p.buffer).img gt $
                      state.d.(state.p.buffer).saturation,count)
            if count ne 0 then (*state.d.(state.p.buffer).mask)[z] = 1.0
            
           
        endif else  *state.d.(state.p.buffer).mask = $
          intarr(state.d.(state.p.buffer).ncols,$
                 state.d.(state.p.buffer).nrows)

        ximgtool_bytsclimg
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE

        
    end
    'Rotation': begin

       case event.value-1 of 
           
           0: rot=7

           1: rot=5

           2: rot=1
           
           3: rot=2

           4: rot=3
           
           5: rot=4

           6: rot=6

       endcase

       *state.d.(state.p.buffer).img = $
         rotate(*state.d.(state.p.buffer).img,rot)

       *state.d.(state.p.buffer).mask = $
         rotate(*state.d.(state.p.buffer).mask,rot)


       s = size(*state.d.(state.p.buffer).img)
       state.d.(state.p.buffer).ncols = s[1]
       state.d.(state.p.buffer).nrows = s[2]

       state.d.(state.p.buffer).cenpix = [state.d.(state.p.buffer).ncols,$
                                          state.d.(state.p.buffer).nrows]/2.-$
         0.5

       ximgtool_bytsclimg
       ximgtool_mkplotimg
       ximgtool_plotupdate,/ERASE
       
   end

   'Saturation Level': begin

       sat = cfld(state.w.saturation_fld,4,/EMPTY,CANCEL=cancel)
       if cancel then return
       state.d.(state.p.buffer).saturation = sat

       if state.p.satpixels then begin

           *state.d.(state.p.buffer).mask = $
             intarr(state.d.(state.p.buffer).ncols,$
                    state.d.(state.p.buffer).nrows)

           z = where(*state.d.(state.p.buffer).img gt sat,count)
           if count ne 0 then (*state.d.(state.p.buffer).mask)[z] = 1.0

           ximgtool_bytsclimg
           ximgtool_mkplotimg
           ximgtool_plotupdate
           
       endif

   end

    'Zoom In': begin

        state.d.(state.p.buffer).zoom = state.d.(state.p.buffer).zoom + 1
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE

    end

    'Zoom Out': begin

        state.d.(state.p.buffer).zoom = state.d.(state.p.buffer).zoom - 1
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE


    end

    'Zoom 1': begin

        state.d.(state.p.buffer).zoom = 0.0
        ximgtool_mkplotimg
        ximgtool_plotupdate,/ERASE

    end

    'CP': ximgtool_mkcp

    else:

endcase

end
;
;******************************************************************************
;
pro ximgtool_plotwinevent,event

common ximgtool_state

widget_control, event.id, GET_UVALUE=uvalue

if strtrim(tag_names(event,/STRUCTURE_NAME),2) eq 'WIDGET_TRACKING' then begin

    if event.enter eq 0 then widget_control, state.w.keyboard, SENSITIVE=0
    goto, cont
    
endif

;  If not, set the keyboard focus and active window.

widget_control, state.w.keyboard, /INPUT_FOCUS, /SENSITIVE
wset, state.p.plotwin_wid
;device, copy=[0,0,state.p.plotwin_size[0],$
;              state.p.plotwin_size[1],0,0,state.p.pixmap_wid]

;  Load the current color table

tvlct,state.d.(state.p.buffer).red,state.d.(state.p.buffer).green,$
  state.d.(state.p.buffer).blue,state.p.color_bottom
ximgtool_color,BRIGHTNESS=state.d.(state.p.buffer).brightness,$
  CONTRAST=state.d.(state.p.buffer).contrast

!x = state.p.xscale
!y = state.p.yscale
!p = state.p.pscale

if TAG_NAMES(event,/STRUCTURE) eq 'WIDGET_TIMER' then begin

    if state.p.firstclick eq 1 then begin

        state.d.(state.p.buffer).cenpix=(convert_coord(state.p.tmpdevpos[0],$
                                                       state.p.tmpdevpos[1],$
                                                       /DEVICE,/TO_DATA))[0:1]
        state.p.firstclick=0
        ximgtool_mkplotimg
        ximgtool_plotupdate,/CENTER,/ERASE

        if xregistered('ximgtool_cp') then begin

            string='Center Pix:  '+$
              strjoin(strtrim(state.d.(state.p.buffer).cenpix,2),',  ')
            widget_control, state.w.cenpix_lb,SET_VALUE=string

        endif
        goto, cont

    endif

endif else begin

    if event.type eq 2 then begin

;  Update cursor position

        pix  = (convert_coord(event.x,event.y,/DEVICE,/TO_DATA))[0:1]
        zpix = roundgt(pix)
        

        if pix[0] gt state.d.(state.p.buffer).ncols-0.5 or $
          pix[0] lt -0.5 or pix[1] gt state.d.(state.p.buffer).nrows-0.5 $
          or pix[1] lt -0.5 then z = 0. else $
          z = (*state.d.(state.p.buffer).img)[zpix[0],zpix[1]]      
        
        string = strjoin([string(pix,format='(f6.1)'), $
                          string(z)],' ')
        
        widget_control, state.w.track,SET_VALUE=string
        

    endif

    case state.p.cursormode of 

        'Color': begin

            if event.release gt 1 then begin

                state.p.cursormode = 'None'
                goto, cont
                
            endif

            ximgtool_color,event
            ximgtool_plotupdate

        end
        
        'Line Cut': begin

            if event.press eq 1 and event.type eq 0 then begin

                state.p.reg[*,0] = [event.x,event.y]
                goto, cont

            endif
            if event.release eq 1 and event.type eq 1 then begin

                state.p.reg[*,1] = [event.x,event.y]
                wset, state.p.pixmap_wid
                plots,[state.p.reg[0,0],state.p.reg[0,1]],$
                  [state.p.reg[1,0],state.p.reg[1,1]],/DEVICE,color=3,THICK=2
                state.p.cursormode = 'None'
                ximgtool_linecut
                goto, cont

            endif 
            if finite(state.p.reg[0]) then begin

                wset, state.p.plotwin_wid
                device, copy=[0,0,state.p.plotwin_size[0],$
                              state.p.plotwin_size[1],0,0,state.p.pixmap_wid]
                plots,[state.p.reg[0,0],event.x],$
                  [state.p.reg[1,0],event.y],/DEVICE,color=3,THICK=2
               
            endif

        end

        'None': begin

            if event.press gt 1 then begin
                
                state.p.cursormode = 'Color'
                goto, cont
                
            endif
            if event.release gt 1 then begin
                
                state.p.cursormode = 'None'
                goto, cont
                
            endif
            
            if event.type eq 0 and state.p.firstclick eq 0 then begin
            
                state.p.tmpdevpos = [event.x,event.y]
                state.p.firstclick = 1
                widget_control, event.id, TIMER=state.d.dblclick
                goto, cont
                
            endif
            
            if event.type eq 0 and state.p.firstclick eq 1 then begin
                
                state.d.(state.p.buffer).zoom = state.d.(state.p.buffer).zoom+1
            state.d.(state.p.buffer).cenpix=(convert_coord(event.x,event.y,$
                                                           /DEVICE,$
                                                           /TO_DATA))[0:1]
                state.p.firstclick=0
                ximgtool_mkplotimg
                ximgtool_plotupdate,/CENTER,/ERASE
                goto, cont
                
            endif
            
        end

    endcase

endelse

cont:


end
;
;******************************************************************************
;
pro ximgtool_resize, event

common ximgtool_state

widget_control, state.w.ximgtool_base, TLB_GET_SIZE=size

state.p.plotwin_size[0]=size[0]-state.p.winbuffer[0]
state.p.plotwin_size[1]=size[1]-state.p.winbuffer[1]

widget_control, state.w.plotwin, DRAW_XSIZE=state.p.plotwin_size[0]
widget_control, state.w.plotwin, DRAW_YSIZE=state.p.plotwin_size[1]

wset, state.p.plotwin_wid
erase

wdelete,state.p.pixmap_wid
window, /FREE, /PIXMAP,XSIZE=state.p.plotwin_size[0],$
  YSIZE=state.p.plotwin_size[1]
state.p.pixmap_wid = !d.window

;  Create new color bar array

widget_control, state.w.cbarwin, DRAW_XSIZE=state.p.plotwin_size[0]

;  Make color bar array

b = cmcongrid( findgen(state.p.ncolors-state.p.color_bottom-1), $
               state.p.plotwin_size[0]) + state.p.color_bottom
c = replicate(1,10)
*state.d.cbarr = b # c

;  Redisplay everything

ximgtool_mkplotimg
ximgtool_plotupdate,/ERASE


end
;
;******************************************************************************
;
; ------------------------------Main Program-------------------------------- 
;
;******************************************************************************
;
pro ximgtool,img,hdr,MIN=min,MAX=max,SCALING=scaling,AUTORANGE=autorange,$
             ROTATION=rotation,GROUP_LEADER=group_leader,WID=wid,$
             NOUPDATE=noupdate,MASK=mask,STDIMAGE=stdimage,CANCEL=cancel

cancel = 0

common ximgtool_state

;  Start widget if neces

if not xregistered('ximgtool') then ximgtool_startup,STDIMAGE=stdimage,$
  GROUP_LEADER=group_leader

wid = state.p.plotwin_wid

;  Get Image

if n_params() gt 0 then begin

    if size(img,/TYPE) eq 7 then begin

        widget_control, state.w.ximgtool_base,$
          TLB_SET_TITLE='Ximgtool - '+strtrim(img,2)   
        image = readfits(img,hdr)

    endif else image = img

;  Make sure the image is 2D

;    cancel = badparmc(image,'Image',[1,2,3,4,5],2,CALLER='Ximgtool')
    if cancel then return

    if n_params() eq 2 then begin

;        cancel = badparmc(hdr,'Hdr',7,1,CALLER='Ximgtool')
        if cancel then return

    endif

endif else image = findgen(1024,1024)

;   Check for ROTATION keyword

if keyword_set(ROTATION) then image = rotate(temporary(image),rotation)

;  Load image and header

*state.d.(0).img = image

if n_params() eq 2 or size(img,/TYPE) eq 7 then *state.d.(0).hdr = hdr

;  Get image info 

imginfo,image,ncols,nrows,dummy,imin,imax,CANCEL=cancel
if cancel then return

if not keyword_set(NOUPDATE) then begin
      
    state.d.(0).min = imin
    state.d.(0).max = imax
    
    state.d.(0).ncols  = ncols
    state.d.(0).nrows  = nrows

    state.d.(0).scaling = 0
    
;  Check for user input keywords
    
    if not keyword_set(MIN) and not keyword_set(MAX) and $
      not keyword_set(AUTORANGE) then ximgtool_imgrange,0
    
    if keyword_set(MIN) then state.d.(0).min = min
    if keyword_set(MAX) then state.d.(0).max = max
    if keyword_set(AUTORANGE) then ximgtool_imgrange,autorange

    if keyword_set(SCALING) then state.d.(0).scaling = scaling

;  Get center pixel 

    state.d.(0).cenpix = [state.d.(0).ncols,state.d.(0).nrows]/2.-0.5
    
endif

*state.d.(0).mask = (n_elements(mask) eq 0) ? intarr(ncols,nrows):mask

;  Plot it up

ximgtool_bytsclimg
ximgtool_mkplotimg
ximgtool_plotupdate,ERASE=(keyword_set(NOUPDATE) eq 1) ? 0:1


end
