;+
; NAME:
;     xsmoothspec
;    
; PURPOSE:
;     Smooths SpeX spectrum with a gaussian or Savitzky-Golay kernel.
;    
; CATEGORY:
;     Widget
;
; CALLING SEQUENCE:
;     xsmoothspec
;    
; INPUTS:
;     None
;    
; OPTIONAL INPUTS:
;     None
;
; KEYWORD PARAMETERS:
;     None
;     
; OUTPUTS:
;     Write SpeX spectral FITS file to disk
;
; OPTIONAL OUTPUTS:
;     None
;
; COMMON BLOCKS:
;     None
;
; SIDE EFFECTS:
;     None
;
; RESTRICTIONS:
;     None
;
; PROCEDURE:
;     
; EXAMPLE:
;     
; MODIFICATION HISTORY:
;     2001-09-29 - Written by M. Cushing, Institute for Astronomy, UH
;     2002-07-23 - Added the Savitzky-Golay option, M. Cushing
;-

;
;******************************************************************************
;
; ------------------------------Event Handlers-------------------------------- 
;
;******************************************************************************
;
pro xsmoothspec_event, event

widget_control, event.id,  GET_UVALUE = uvalue
if uvalue eq 'Quit' then begin

    widget_control, event.top, /DESTROY
    goto, getout

endif


widget_control, event.top, GET_UVALUE = state, /NO_COPY
widget_control, /HOURGLASS

case uvalue of

    'Help': xsmoothspec_help, state

    'Load Spectrum': xsmoothspec_loadspec,state

    'Output Format': begin

        if event.value eq 'Text' then state.r.textoutput=event.select
        if event.value eq 'FITS' then state.r.fitsoutput=event.select

    end

    'Path Button': begin

        path= dialog_pickfile(/DIRECTORY,$
                              DIALOG_PARENT=state.w.xsmoothspec_base,$
                              TITLE='Select Path',/MUST_EXIST)
        if path ne '' then begin
            
            path = cpath(path,WIDGET_ID=state.w.xsmoothspec_base,CANCEL=cancel)
            if cancel then return
            widget_control,state.w.path_fld[1],SET_VALUE = path
            setfocus,state.w.path_fld

        endif

    end

    'Smooth Spectra': xsmoothspec_smoothspec,state

    'Smooth Type': begin

        widget_control, state.w.sg_base, MAP=0
        widget_control, state.w.gs_base, MAP=0

        if event.value eq 'Savitzky-Golay' then begin

            state.r.smoothtype = event.value
            widget_control, state.w.sg_base, MAP=1

        endif
        if event.value eq 'Change <R>' then begin

            state.r.smoothtype = event.value
            widget_control, state.w.gs_base, MAP=1
            
        endif

    end

    'Spectra File Button': begin
                
        path = cfld(state.w.path_fld,7,CANCEL=cancel)
        if cancel then return
        if path ne '' then path = cpath(path,$
                                        WIDGET_ID=state.w.xsmoothspec_base,$
                                        CANCEL=cancel)

        fullpath = dialog_pickfile(DIALOG_PARENT=state.w.xsmoothspec_base,$
                                   PATH=path,/MUST_EXIST,FILTER='*.fits')
        
        if fullpath ne '' then begin
            
            widget_control,state.w.specfiles_fld[1],$
              set_value = strmid(fullpath,$
                                 strpos(fullpath,state.w.dirsep,/REVERSE_S)+1)
            
        endif

    end

    'Spectra Files Field': setfocus,state.w.outfile_fld

    else:

endcase

;  Put state variable into the user value of the top level base.
 
cont: 
widget_control, state.w.xsmoothspec_base, SET_UVALUE=state, /NO_COPY
getout:

end
;
;******************************************************************************
;
pro xsmoothspechelp_event,event

widget_control, event.top, /DESTROY

end
;
;******************************************************************************
;
; ----------------------------Support procedures------------------------------ 
;
;******************************************************************************
;
pro xsmoothspec_cleanup,xsmoothspec_base

widget_control, xsmoothspec_base, GET_UVALUE = state, /NO_COPY

if n_elements(state) ne 0 then begin

    ptr_free, state.r.hdr
    ptr_free, state.r.orders
    ptr_free, state.r.spec

endif
state = 0B

end
;
;******************************************************************************
;
pro xsmoothspec_help,state

getfonts,buttonfont,textfont

if not xregistered('xsmoothspec_help') then begin

    help_base = widget_base(GROUP_LEADER=state.w.xsmoothspec_base,$
                            /COLUMN, $
                            TITLE='Xsmoothspec Help')
    
    openr, lun, filepath('xsmoothspec_helpfile.txt',$
                         ROOT_DIR=state.r.packagepath,SUBDIR='helpfiles'),$
      /GET_LUN
    nlines = numlines(filepath('xsmoothspec_helpfile.txt',$
                               ROOT_DIR=state.r.packagepath,$
                               SUBDIR='helpfiles'))
    array = strarr(nlines)
    readf, lun, array
    free_lun, lun


    help_text = widget_text(help_base, $
                            /SCROLL, $
                            VALUE=array, $
                            XSIZE=80, $
                            YSIZE=24)
    
    quit = widget_button(help_base,$
                         VALUE='Done',$
                         FONT=buttonfont,$
                         UVALUE='Done')
      
    centertlb,help_base

    widget_control, help_base, /REALIZE
 
; Start the Event Loop. This will be a non-blocking program.
       
    XManager, 'xsmoothspec_help', $
      help_base, $
      /NO_BLOCK,$
      EVENT_HANDLER='xsmoothspechelp_event'
    
endif

End
;
;******************************************************************************
;
pro xsmoothspec_loadspec,state

path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return
path = cpath(path,WIDGET_ID=state.w.xsmoothspec_base,CANCEL=cancel)
if cancel then return

file = cfld(state.w.specfiles_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return
file = cfile(path+file,WIDGET_ID=state.w.xsmoothspec_base,CANCEL=cancel)
if cancel then return

;  Read spectra

readspec,file,spec,hdr,obsmode,start,stop,norders,naps,orders,$
  xunits,yunits,slith_pix,slith_arc,slitw_pix,slitw_arc,$
  airmass,CANCEL=cancel

*state.r.hdr    = hdr
*state.r.spec   = spec
state.r.norders = norders
state.r.naps    = naps
state.r.start   = start
state.r.stop    = stop
*state.r.orders = orders
state.r.xunits  = xunits
state.r.yunits  = yunits
state.r.slitw_pix = slitw_pix

widget_control, state.w.sgwin_fld[1],$
  SET_VALUE=string(slitw_pix*2.5,FORMAT='(f4.1)')

if obsmode eq 'LongXD1.9' or $
   obsmode eq 'LongXD2.1' or $
   obsmode eq 'LongXD2.3' then obsmode = 'LongXD'

case obsmode of 

    'ShortXD': begin

        state.r.resperpix = 4000.

;  Determine the resolution (max) if smoothed by a 1 pixel gaussian

        state.r.maxres = 4000./sqrt(slitw_pix^2+1)
        widget_control, state.w.maxres_lbl,SET_VALUE='< '+$
          string(state.r.maxres,FORMAT='(i4.4)')

    end

    'LongXD': begin
    
        state.r.resperpix = 5000.

;  Determine the resolution (max) if smoothed by a 1 pixel gaussian

        state.r.maxres = 5000./sqrt(slitw_pix^2+1)
        widget_control, state.w.maxres_lbl,SET_VALUE='< '+$
          string(state.r.maxres,FORMAT='(i4.4)')

    end

    else: 

endcase

end
;
;******************************************************************************
;
pro xsmoothspec_smoothspec,state


path = cfld(state.w.path_fld,7,CANCEL=cancel)
if cancel then return
path = cpath(path,WIDGET_ID=state.w.xsmoothspec_base,CANCEL=cancel)
if cancel then return

ofile = cfld(state.w.outfile_fld,7,/EMPTY,CANCEL=cancel)
if cancel then return

spec = (*state.r.spec)


if state.r.smoothtype eq 'Savitzky-Golay' then begin

    res = state.r.resperpix/state.r.slitw_pix

    sgwin = cfld(state.w.sgwin_fld,4,/EMPTY,CANCEL=cancel)
    if cancel then return

    sgdeg = cfld(state.w.sgdeg_fld,4,/EMPTY,CANCEL=cancel)
    if cancel then return
    
    sgdeg = crange(sgdeg,sgwin,'SG Degree',/KLT,$
                   WIDGET_ID=state.w.xsmoothspec_base,CANCEL=cancel)
    if cancel then return

    print, ' '
    print, 'Performing a Savitzky-Golay smoothing with a window of '+$
      strtrim(sgwin,2)
    print, ' and a degree of '+strtrim(sgdeg,2)+'.'
    print, ' '

    for i = 0, state.r.norders-1 do begin
        
        for j = 0, state.r.naps-1 do begin
            
            z = where(finite((*state.r.spec)[*,1,i*state.r.naps+j]) eq 1)

            cflux  = (*state.r.spec)[z,1,i*state.r.naps+j]
            cerror = (*state.r.spec)[z,2,i*state.r.naps+j]
            
            nan = where(finite(cflux) eq 0,cnt)

            cflux = savitzky_golay(cflux,sgwin,DEGREE=sgdeg,ERROR=cerror)

            if cnt ne 0 then begin

                cflux[nan]  = !values.f_nan
                cerror[nan] = !values.f_nan

            endif

            spec[z,1,i*state.r.naps+j] = cflux
            spec[z,2,i*state.r.naps+j] = cerror
            
        endfor
        
    endfor

    history='These spectra have been convolved with a Savitzky-Golay filter'+$
      ' of width '+strtrim(sgwin,2)+' and degree '+strtrim(sgdeg,2)+'.'

endif else begin

    res = cfld(state.w.res_fld,4,/EMPTY,CANCEL=cancel)
    if cancel then return
    res = crange(res,[1,state.r.maxres],'<R>',/KLE,$
                 WIDGET_ID=state.w.xsmoothspec_base,CANCEL=cancel)
    if cancel then return
    print, state.r.resperpix
    FWHM_final  = state.r.resperpix/res
    FWHM_kernel = sqrt(FWHM_final^2 - state.r.slitw_pix^2)

    print, ' '
    print, 'Smoothing with a '+string(FWHM_kernel,format='(f3.1)')+$
      ' gaussian for a new resolution of '+strtrim(res,2)+'.'
    print, ' '

    for i = 0, state.r.norders-1 do begin
        
        for j = 0, state.r.naps-1 do begin
            
            z = where(finite((*state.r.spec)[*,1,i*state.r.naps+j]) eq 1)

            cflux  = (*state.r.spec)[z,1,i*state.r.naps+j]
            cerror = (*state.r.spec)[z,2,i*state.r.naps+j]
            
            nan = where(finite(cflux) eq 0,cnt)

            convolvespec,findgen(state.r.stop-state.r.start+1),$
                         cflux,FWHM_kernel,cflux,cerror,$
                         ERROR=cerror,CANCEL=cancel
            
            if cnt ne 0 then begin

                cflux[nan]  = !values.f_nan
                cerror[nan] = !values.f_nan

            endif
            
            spec[z,1,i*state.r.naps+j] = cflux
            spec[z,2,i*state.r.naps+j] = cerror
            
        endfor
        
    endfor

    history = 'These spectra have been convolved with a Gaussain of FWHM='+$
      strcompress(FWHM_kernel,/RE)+'.'

endelse

;  Write results to disk

hdr = *state.r.hdr

length = strlen(history)
loop = ceil(float(length)/65.)
for i = 0, loop-1 do begin
    
    hist = strmid(history,65*i,65)
    fxaddpar,hdr,'HISTORY',hist
    
endfor
fxaddpar,hdr,'RES',res


if state.r.fitsoutput then begin

    writefits,path+ofile+'.fits',spec,hdr
    xvspec,path+ofile+'.fits'

endif

if state.r.textoutput then begin

    npix = fxpar(hdr,'NAXIS1')
    openw,lun,path+ofile+'.txt', /GET_LUN

    for i = 0, n_elements(hdr)-1 do printf, lun, hdr[i]

    for i = 0, npix[0]-1 do begin

        printf, lun,  strjoin( reform(spec[i,*,*],$
                                      3*state.r.naps*state.r.norders),'  ' )

    endfor
    close, lun
    free_lun, lun

endif

end
;
;******************************************************************************
;
pro xsmoothspec

getosinfo,dirsep,strsep

last        = strpos(!path,'Spextool')
first       = strpos(!path,strsep,last,/REVERSE_SEARCH)
packagepath = strmid(!path,first+1,last-first+8)

;  Load color table 

mkct

;  Set the fonts

getfonts,buttonfont,textfont

;  Build three structures which will hold important info.

w = {dirsep:dirsep,$
     filename_fld:[0L,0L],$
     maxres_lbl:0L,$
     path_fld:[0L,0L],$
     outfile_fld:[0L,0L],$
     sg_base:0L,$
     sgwin_fld:[0L,0L],$
     sgdeg_fld:[0L,0L],$
     gs_base:0L,$
     gs_fld:[0L,0L],$
     res_fld:[0L,0L],$
     specfiles_fld:[0L,0L],$
     xsmoothspec_base:0L}

r = {fitsoutput:1,$
     hdr:ptr_new(fltarr(2)),$
     maxres:0.,$
     norders:0,$
     naps:0,$
     orders:ptr_new(fltarr(2)),$
     packagepath:packagepath,$
     resperpix:0.,$
     sgdeg:2,$
     sgwin:0.0,$
     slitw_pix:0.,$
     smoothtype:'Savitzky-Golay',$
     spec:ptr_new(fltarr(2)),$
     start:0,$
     stop:0,$
     textoutput:0,$
     xunits:'',$
     yunits:''}

state = {w:w,r:r}

;  Build the widget.

state.w.xsmoothspec_base = widget_base(TITLE='Xsmoothspec', $
                                       /COLUMN,$
                                       EVENT_PRO='xsmoothspec_event')

   quit_button = widget_button(state.w.xsmoothspec_base,$
                               FONT=buttonfont,$
                               VALUE='Quit',$
                               UVALUE='Quit')

   text = widget_text(state.w.xsmoothspec_base,$
                      value='To be used on unsmoothed data only.')


   box1_base = widget_base(state.w.xsmoothspec_base,$
                           /COLUMN,$
                           FRAME=1)
   
      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
      
         button = widget_button(row,$
                                FONT=buttonfont,$
                                VALUE='Path',$
                                UVALUE='Path Button')
         
         field = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE=':',$
                               UVALUE='Path Field',$
                               XSIZE=30,$
                               EVENT_PRO='xsmoothspec_event',$
                               /CR_ONLY,$
                               TEXTID=textid)
         state.w.path_fld = [field,textid]               
         
      row = widget_base(box1_base,$
                        /ROW,$
                        /BASE_ALIGN_CENTER)
      
         button = widget_button(row,$
                                FONT=buttonfont,$
                                VALUE='Spectra File',$
                                UVALUE='Spectra File Button')
         
         field = coyote_field2(row,$
                               LABELFONT=buttonfont,$
                               FIELDFONT=textfont,$
                               TITLE=':',$
                               UVALUE='Spectra Files Field',$
                               XSIZE=25,$
                               EVENT_PRO='xsmoothspec_event',$
                               /CR_ONLY,$
                               TEXTID=textid)
         state.w.specfiles_fld = [field,textid]

      load_button = widget_button(box1_base,$
                                  FONT=buttonfont,$
                                  VALUE='Load Spectrum',$
                                  UVALUE='Load Spectrum')

   box2_base = widget_base(state.w.xsmoothspec_base,$
                           /COLUMN,$
                           FRAME=1)

     smooth_bg = cw_bgroup(box2_base,$
                           FONT=buttonfont,$
                           ['Savitzky-Golay','Change <R>'],$
                           /ROW,$
                           /RETURN_NAME,$
                           /NO_RELEASE,$
                           /EXCLUSIVE,$
                           LABEL_LEFT='',$
                           UVALUE='Smooth Type',$
                           SET_VALUE=0)

     row = widget_base(box2_base)

        state.w.sg_base = widget_base(row,$
                                      /ROW,$
                                      /BASE_ALIGN_CENTER)

           window = coyote_field2(state.w.sg_base,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE='SG Window:',$
                                  UVALUE='SG Window',$
                                  XSIZE=7,$
                                  TEXTID=textid)
           state.w.sgwin_fld = [field,textid] 

           window = coyote_field2(state.w.sg_base,$
                                  LABELFONT=buttonfont,$
                                  FIELDFONT=textfont,$
                                  TITLE='SG Degree:',$
                                  UVALUE='SG Degree',$
                                  VALUE='2',$
                                  XSIZE=7,$
                                  TEXTID=textid)
           state.w.sgdeg_fld = [field,textid] 
        
        state.w.gs_base = widget_base(row,$
                                      /ROW,$
                                      /BASE_ALIGN_CENTER,$
                                      MAP=0)
        
           field = coyote_field2(state.w.gs_base,$
                                 LABELFONT=buttonfont,$
                                 FIELDFONT=textfont,$
                                 TITLE='<R> = ',$
                                 UVALUE='<R>',$
                                 XSIZE=10,$
                                 EVENT_PRO='xsmoothspec_event',$
                                 /CR_ONLY,$
                                 TEXTID=textid)
           state.w.res_fld = [field,textid] 

           state.w.maxres_lbl = widget_label(state.w.gs_base,$
                                             /DYNAMIC_RESIZE,$
                                             FONT=buttonfont,$
                                             VALUE=' < ')
           
   box3_base = widget_base(state.w.xsmoothspec_base,$
                           /COLUMN,$
                           FRAME=1)

      outformat_bg = cw_bgroup(box3_base,$
                               FONT=buttonfont,$
                               ['FITS','Text'],$
                               /ROW,$
                               /RETURN_NAME,$
                               /NONEXCLUSIVE,$
                               LABEL_LEFT='Output Format:',$
                               UVALUE='Output Format',$
                               SET_VALUE=[1,0])
      
      field = coyote_field2(box3_base,$
                            LABELFONT=buttonfont,$
                            FIELDFONT=textfont,$
                            TITLE='Output File:',$
                            UVALUE='Output File',$
                            XSIZE=25,$
                            EVENT_PRO='xsmoothspec_event',$
                            /CR_ONLY,$
                            TEXTID=textid)
      state.w.outfile_fld = [field,textid]

      
      smooth_button = widget_button(box3_base,$
                                    FONT=buttonfont,$
                                    VALUE='Smooth Spectra',$
                                    UVALUE='Smooth Spectra')

   help_button = widget_button(state.w.xsmoothspec_base,$
                               FONT=buttonfont,$
                               VALUE='Help',$
                               UVALUE='Help')

; Get things running.  Center the widget using the Fanning routine.

centertlb,state.w.xsmoothspec_base

widget_control, state.w.xsmoothspec_base, /REALIZE

; Start the Event Loop. This will be a non-blocking program.

XManager, 'xsmoothspec', $
  state.w.xsmoothspec_base, $
  /NO_BLOCK,$
  CLEANUP='xsmoothspec_cleanup'

; Put state variable into the user value of the top level base.

widget_control, state.w.xsmoothspec_base, SET_UVALUE=state, /NO_COPY

end






